from unittest import TestCase
from tgt_grease.core import Configuration
from tgt_grease.enterprise.Sources import url_source
import json
import os
import datetime


class TestUrlParser(TestCase):

    def test_url_parser_empty(self):
        source = url_source()
        self.assertFalse(source.parse_source({}))
        self.assertFalse(source.get_data())

    def test_url_parser_mock(self):
        source = url_source()
        conf = Configuration()
        mock = {
            'url': 'https://google.com',
            'status_code': 200,
            'headers': str({'test': 'ver', 'test1': 'val'}),
            'body': 'welcome to google'
        }
        fil = open(conf.greaseDir + 'etc' + conf.fs_sep + 'test.mock.url.json', 'w')
        fil.write(json.dumps(mock))
        fil.close()
        mockData = source.mock_data({})
        self.assertEqual(len(mockData), 1)
        self.assertEqual(mock.get('url'), mockData[0].get('url'))
        self.assertEqual(mock.get('status_code'), mockData[0].get('status_code'))
        self.assertEqual(mock.get('headers'), mockData[0].get('headers'))
        self.assertEqual(mock.get('body'), mockData[0].get('body'))
        os.remove(conf.greaseDir + 'etc' + conf.fs_sep + 'test.mock.url.json')

    def test_url_parser_single_source(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 1)
        self.assertEqual(b'http://www.google.com/', Data[0].get('url'))
        self.assertEqual(200, Data[0].get('status_code'))

    def test_url_parser_multiple_source(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'url': ['google.com', 'bing.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 2)

    def test_url_parser_test_hour_good(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'hour': datetime.datetime.utcnow().hour,
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 1)
        self.assertEqual(b'http://www.google.com/', Data[0].get('url'))
        self.assertEqual(200, Data[0].get('status_code'))

    def test_url_parser_test_minute_good(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'minute': datetime.datetime.utcnow().minute,
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 1)
        self.assertEqual(b'http://www.google.com/', Data[0].get('url'))
        self.assertEqual(200, Data[0].get('status_code'))

    def test_url_parser_test_hour_and_minute_good(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'hour': datetime.datetime.utcnow().hour,
            'minute': datetime.datetime.utcnow().minute,
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 1)
        self.assertEqual(b'http://www.google.com/', Data[0].get('url'))
        self.assertEqual(200, Data[0].get('status_code'))

    def test_url_parser_test_hour_bad(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'hour': (datetime.datetime.utcnow() + datetime.timedelta(hours=6)).hour,
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 0)

    def test_url_parser_test_minute_bad(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'minute': (datetime.datetime.utcnow() + datetime.timedelta(minutes=10)).minute + 7,
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 0)

    def test_url_parser_test_hour_and_minute_bad(self):
        source = url_source()
        self.assertTrue(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'url_source',
            'hour': (datetime.datetime.utcnow() + datetime.timedelta(hours=6)).hour,
            'minute': (datetime.datetime.utcnow() + datetime.timedelta(minutes=10)).minute + 7,
            'url': ['google.com'],
            'logic': {}
        }))
        Data = source.get_data()
        self.assertEqual(len(Data), 0)



from tgt_grease.core import GreaseContainer
from tgt_grease.core import ImportTool
from .Configuration import PrototypeConfig
from .BaseSource import BaseSourceClass
from .DeDuplication import Deduplication
from .CentralScheduling import Scheduling
import threading
from psutil import cpu_percent, virtual_memory
from uuid import uuid4


class Scan(object):
    """Scanning class for GREASE Scanner

    This is the model to actually utilize the scanners to parse the configured environments

    Attributes:
        ioc (GreaseContainer): IOC for scanning
        conf (PrototypeConfig): Prototype configuration instance
        impTool (ImportTool): Import Utility Instance
        dedup (Deduplication): Deduplication instance to be used

    """

    def __init__(self, ioc=None):
        if ioc and isinstance(ioc, GreaseContainer):
            self.ioc = ioc
        else:
            self.ioc = GreaseContainer()
        self.conf = PrototypeConfig(self.ioc)
        self.impTool = ImportTool(self.ioc.getLogger())
        self.dedup = Deduplication(self.ioc)
        self.scheduler = Scheduling(self.ioc)

    def Parse(self, source=None, config=None):
        """This will read all configurations and attempt to scan the environment

        This is the primary business logic for scanning in GREASE. This method will use configurations to parse
        the environment and attempt to schedule

        Note:
            If a Source is specified then *only* that source is parsed. If a configuration is set then *only* that
            configuration is parsed. If both are provided then the configuration will *only* be parsed if it is of
            the source provided

        Note:
            **If mocking is enabled**: Deduplication *will not occur*

        Args:
            source (str): If set will only parse for the source listed
            config (str): If set will only parse the specified config

        Returns:
            bool: True unless error

        """
        self.ioc.getLogger().trace("Starting Parse of Environment", trace=True)
        Configuration = self.generate_config_set(source=source, config=config)
        ScanPool = []
        lenConfigs = len(Configuration)
        i = 0
        while i < lenConfigs:
            # ensure we don't swamp the system resources
            cpu = cpu_percent(interval=.1)
            mem = virtual_memory().percent
            if \
                    cpu >= int(self.ioc.getConfig().get('NodeInformation', 'ResourceMax')) or \
                    mem >= int(self.ioc.getConfig().get('NodeInformation', 'ResourceMax')):
                self.ioc.getLogger().trace("Scan sleeping; System resource maximum reached", verbose=True)
                # remove variables
                del cpu
                del mem
                continue
            conf = Configuration[i]
            i += 1
            # ensure no kafka prototypes come into sourcing
            if conf.get('source') == 'kafka':
                continue
            # ensure there is an execution environment
            server, _ = self.scheduler.determineExecutionServer(conf.get('exe_env', 'general'))
            if not server:
                self.ioc.getLogger().warning(
                    'configuration skipped -- execution environment offline',
                    additional={
                        'execution_environment': conf.get('exe_env', 'general'),
                        'configuration': conf.get('name')
                    },
                    notify=True
                )
                continue
            inst = self.impTool.load(conf.get('source', str(uuid4())))
            if not isinstance(inst, BaseSourceClass):
                self.ioc.getLogger().error("Invalid Source [{0}]".format(conf.get('source')), notify=False)
                del inst
                continue
            else:
                t = threading.Thread(
                    target=self.ParseSource,
                    args=(
                        self.ioc,
                        inst,
                        conf,
                        self.dedup,
                        self.scheduler,
                    ),
                    name="GREASE SOURCING THREAD [{0}]".format(conf.get('name'))
                )
                t.daemon = True
                t.start()
                ScanPool.append(t)
        # wait for threads to finish out
        while len(ScanPool) > 0:
            self.ioc.getLogger().trace("Total current scan threads [{0}]".format(len(ScanPool)), trace=True)
            threads_final = []
            for thread in ScanPool:
                if thread.isAlive():
                    threads_final.append(thread)
            ScanPool = threads_final
            self.ioc.getLogger().trace("Total current scan threads [{0}]".format(len(ScanPool)), trace=True)
        self.ioc.getLogger().trace("Scanning Complete".format(len(ScanPool)), trace=True)
        return True

    @staticmethod
    def ParseSource(ioc, source, configuration, deduplication, scheduler):
        """Parses an individual source and attempts to schedule it

        Args:
            ioc (GreaseContainer): IoC Instance
            source (BaseSourceClass): Source to parse
            configuration (dict): Prototype configuration to use
            deduplication (Deduplication): Dedup engine instance
            scheduler (Scheduling): Central Scheduling instance

        Returns:
            None: Meant to be run in a thread

        """
        try:
            # If mock mode enabled
            if ioc.getConfig().get('Sourcing', 'mock'):
                data = source.mock_data(configuration)
            # else actually do sourcing
            else:
                if source.parse_source(configuration):
                    # deduplicate data
                    data = deduplication.Deduplicate(
                        data=source.get_data(),
                        source=configuration.get('source'),
                        configuration=configuration.get('name', str(uuid4())),
                        threshold=source.deduplication_strength,
                        expiry_hours=source.deduplication_expiry,
                        expiry_max=source.deduplication_expiry_max,
                        collection='Dedup_Sourcing',
                        field_set=source.field_set
                    )
                else:
                    ioc.getLogger().warning(
                        "Source [{0}] parsing failed".format(configuration.get('source')),
                        notify=False
                    )
                    data = []
            if len(data) > 0:
                if scheduler.scheduleDetection(configuration.get('source'), configuration.get('name'), data):
                    ioc.getLogger().info(
                        "Data scheduled for detection from source [{0}]".format(configuration.get('source')),
                        trace=True
                    )
                    del source
                else:
                    ioc.getLogger().error("Scheduling failed for source document!", notify=False)
                    del source
            else:
                ioc.getLogger().trace("Length of data was empty; was not scheduled", trace=True)
                del source
        except BaseException as e:
            ioc.getLogger().error(
                "Failed parsing message got exception! Configuration [{0}] Got [{1}]".format(configuration, e)
            )
            del source

    def generate_config_set(self, source=None, config=None):
        """Examines configuration and returns list of configs to parse

        Note:
            If a Source is specified then *only* that source is parsed. If a configuration is set then *only* that
            configuration is parsed. If both are provided then the configuration will *only* be parsed if it is of
            the source provided

        Args:
            source (str): If set will only parse for the source listed
            config (str): If set will only parse the specified config

        Returns:
            list[dict]: Returns Configurations to Parse for data

        """
        ConfigList = []
        if source and config:
            if self.conf.get_config(config).get('source') == source:
                ConfigList.append(self.conf.get_config(config))
                return ConfigList
            else:
                self.ioc.getLogger().warning(
                    "Configuration [{0}] Not Found With Correct Source [{1}]".format(config, source),
                    trace=True,
                    notify=False
                )
        elif source and not config:
            if source in self.conf.get_sources():
                for configuration in self.conf.get_source(source):
                    ConfigList.append(configuration)
                return ConfigList
            else:
                self.ioc.getLogger().warning(
                    "Source not found in Configuration [{0}]".format(source),
                    trace=True,
                    notify=False
                )
        elif not source and config:
            if self.conf.get_config(config):
                ConfigList.append(self.conf.get_config(config))
                return ConfigList
            else:
                self.ioc.getLogger().warning(
                    "Config not found in Configuration [{0}]".format(config),
                    trace=True,
                    notify=False
                )
        else:
            ConfigList = self.conf.getConfiguration().get('raw')
        return ConfigList



from tgt_grease.core.Types import Command
from logging import ERROR
import platform
import sys
import os
import subprocess
import datetime
from .Daemon import DaemonProcess
if platform.system().lower().startswith("win"):
    import win32serviceutil
    import win32service
    import win32event
    import servicemanager
    import socket


    class AppServerSvc (win32serviceutil.ServiceFramework):
        """Windows Service Configuration"""
        _svc_name_ = "GreaseDaemon"
        _svc_display_name_ = "GREASE Daemon Server"
        _svc_description_ = "GREASE Async Daemon Server for Automation Work"
        hWaitStop = win32event.CreateEvent(None, 0, 0, None)

        def __init__(self, args):
            win32serviceutil.ServiceFramework.__init__(self, args)
            self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)
            socket.setdefaulttimeout(60)

        def SvcStop(self):
            self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
            win32event.SetEvent(self.hWaitStop)

        def SvcDoRun(self):
            servicemanager.LogMsg(
                servicemanager.EVENTLOG_INFORMATION_TYPE,
                servicemanager.PYS_SERVICE_STARTED,
                (self._svc_name_, '')
            )
            self.main()

        def main(self):
            if 'install' not in sys.argv:
                inst = Daemon()
                inst.run()
            else:
                return

        def start(self):
            self.SvcDoRun()

        def stop(self):
            self.SvcStop()

        def restart(self):
            self.SvcStop()
            self.SvcDoRun()


MacOSPListFile = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
    <key>Label</key>
    <string>net.grease.daemon</string>
    <key>ProgramArguments</key>
    <array>
        <string>{0}</string>
        <string>{1}/grease</string>
        <string>daemon</string>
        <string>run</string>
    </array>
    <key>RunAtLoad</key>
    <true />
</dict>
</plist>
""".format(sys.executable, os.sep.join(sys.executable.split(os.sep)[:-1]))

SystemdFile = """
[Unit]
Description=GREASE Daemon Service
After=syslog.target

[Service]
Type=simple
WorkingDirectory=/opt/grease
PIDFile=/var/run/grease.pid
ExecStart={0} {1}/grease daemon run
StandardOutput=syslog
StandardError=syslog

[Install]
WantedBy=multi-user.target
""".format(sys.executable, os.sep.join(sys.executable.split(os.sep)[:-1]))


class Daemon(Command):
    """Daemon Class for the daemon"""

    __author__ = "James E. Bell Jr."
    __version__ = "2.0.0"
    purpose = "Control Daemon Processing in GREASE"
    help = """
    Provide simple abstraction for daemon operations in GREASE
    
    Args:
        install
            install the daemon on the system 
        start
            start the daemon
        stop
            stop the daemon
        run
            run the daemon in the foreground    
            
        --loop:<int>
            Number of cycles to run the daemon
        --foreground
            If provided this will print log messages into the foreground
    
    """

    def __init__(self):
        super(Daemon, self).__init__()

    def execute(self, context):
        if context.get('foreground'):
            self.ioc.getLogger().foreground = True
        if 'install' in context.get('grease_other_args'):
            return bool(self.install())
        elif 'start' in context.get('grease_other_args'):
            return bool(self.start())
        elif 'stop' in context.get('grease_other_args'):
            return bool(self.stop())
        elif 'run' in context.get('grease_other_args'):
            try:
                return self.run(context.get('loop'))
            except KeyboardInterrupt:
                return True
        else:
            print("Invalid Sub-command here is help data:")
            print(self.help)
            return False

    def install(self):
        """Handle Daemon Installation based on the platform we're working with

        Returns:
            bool: installation success

        """
        global MacOSPListFile, SystemdFile
        plat = platform.system().lower()
        if plat.startswith("win"):
            # Windows
            print("Installing Windows Service")
            win32serviceutil.HandleCommandLine(AppServerSvc, argv=['grease', 'install'])
            return True
        elif plat.startswith("dar"):
            # MacOS
            if not os.path.isfile(self.ioc.getConfig().greaseDir + "/etc/grease.daemon"):
                fil = open(self.ioc.getConfig().greaseDir + "/etc/grease.daemon", "w")
                fil.write(MacOSPListFile)
                fil.close()
            if subprocess.call(
                    [
                        "sudo",
                        "cp",
                        self.ioc.getConfig().greaseDir + "/etc/grease.daemon",
                        "/Library/LaunchDaemons/net.grease.daemon.plist"
                    ]
            ) != 0:
                print("Failed to install LaunchCtl Daemon!")
                print("""
                Please manually install! Do this by placing the file: 
                {0}
                Under the name:
                {1}
                !! Ensure you have the correct permissions setup !!
                """.format(
                        self.ioc.getConfig().greaseDir + "/etc/grease.daemon",
                        "/Library/LaunchDaemons/net.grease.daemon.plist"
                    )
                )
                return False
            return True
        elif plat.startswith("lin"):
            # Linux
            if not os.path.isfile(self.ioc.getConfig().greaseDir + "/etc/grease.daemon"):
                fil = open(self.ioc.getConfig().greaseDir + "/etc/grease.daemon", "w")
                fil.write(SystemdFile)
                fil.close()
            if subprocess.call(
                    [
                        "sudo",
                        "cp",
                        self.ioc.getConfig().greaseDir + "/etc/grease.daemon",
                        "/etc/systemd/system/grease.service"
                    ]
            ) != 0:
                print("Failed to install Systemd Service!")
                print("""
                Please manually install! Do this by placing the file: 
                {0}
                Under the name:
                {1}
                !! Ensure you have the correct permissions setup !!
                """.format(
                        self.ioc.getConfig().greaseDir + "/etc/grease.daemon",
                        "/etc/systemd/system/grease.service"
                    )
                )
                return False
            return True
        else:
            self.ioc.getLogger().error("Unrecognized operating system [{0}]".format(platform))
            print("Unrecognized operating system [{0}]".format(platform))
            return False

    def start(self):
        """Starting the daemon based on platform

        Returns:
            bool: start success

        """
        plat = platform.system().lower()
        if plat.startswith("win"):
            # Windows
            win32serviceutil.HandleCommandLine(AppServerSvc, argv=['grease', 'start'])
            return True
        elif plat.startswith("dar"):
            # MacOS
            if subprocess.call(["sudo", "launchctl", "load", "/Library/LaunchDaemons/net.grease.daemon.plist"]) != 0:
                return False
            return True
        elif plat.startswith("lin"):
            # Linux
            if subprocess.call(["sudo", "systemctl", "start", "grease"]) != 0:
                return False
            return True
        else:
            self.ioc.getLogger().error("Unrecognized operating system [{0}]".format(platform))
            return False

    def stop(self):
        """Stopping the daemon based on the platform

        Returns:
            bool: stop success

        """
        plat = platform.system().lower()
        if plat.startswith("win"):
            # Windows
            win32serviceutil.HandleCommandLine(AppServerSvc, argv=['grease', 'stop'])
            return True
        elif plat.startswith("dar"):
            if subprocess.call(["sudo", "launchctl", "unload", "/Library/LaunchDaemons/net.grease.daemon.plist"]) != 0:
                return False
            # MacOS
            return True
        elif plat.startswith("lin"):
            # Linux
            if subprocess.call(["sudo", "systemctl", "stop", "grease"]) != 0:
                return False
            return True
        else:
            self.ioc.getLogger().error("Unrecognized operating system [{0}]".format(platform))
            return False

    def run(self, loop=None):
        """Actual running of the daemon

        Args:
            loop (int): Amount of cycles the daemon should run for

        Returns:
            bool: Server running state

        """
        daemon = DaemonProcess(self.ioc)
        if not daemon.registered:
            self.ioc.getLogger().critical("Node is not registered!")
            return False
        if not loop:
            rc = 'default'
            while True:
                # Windows SysCall Monitoring
                if platform.system().lower().startswith('win'):
                    if not rc != win32event.WAIT_OBJECT_0:
                        self.ioc.getLogger().debug("Windows Kill Signal Detected! Closing GREASE")
                if not daemon.server():
                    daemon.log_once_per_second("Server Process Failed", ERROR)
                # After all this check for new windows services
                if platform.system().lower().startswith('win'):
                    # Block .5ms to listen for exit sig
                    rc = win32event.WaitForSingleObject(AppServerSvc.hWaitStop, 5)

        else:
            self.ioc.getLogger().debug("Daemon in timed mode")
            runs = 0
            loop = int(loop)
            while runs < loop:
                if not daemon.server():
                    daemon.log_once_per_second("Server Process Failed", ERROR)
                daemon.log_once_per_second(
                    "Daemon Server process complete for second [{0}]".format(datetime.datetime.utcnow().second)
                )
                runs += 1
            if not daemon.drain_jobs(self.ioc.getCollection('SourceData')):
                self.ioc.getLogger().error("Failed to drain jobs", notify=False)
        return True



from tgt_grease.core.Types import Command
from tgt_grease.enterprise.Model import KafkaSource


class Kafka(Command):
    """The Kafka Command

    This class is the ingestion of Kafka topics for GREASE. 

    """

    __author__ = "Ronald Queensen, Dustin Gundrum"
    __version__ = "1.0.0"
    purpose = "Listen to the configured Kafka topics and send the chosen keys to detection"
    help = """
    This command spins up a KafkaConsumer object, and runs KafkaSource.run, which will load all the kafka configs and begin consuming messages.
    
    Args:
        --config:<filename>
            The specific config file you want to parse
        --foreground
            Print Log messages to foreground
    """

    def __init__(self):
        super(Kafka, self).__init__()

    def execute(self, context):
        """Execute method of the Kafka prototype

        Args:
            context (dict): Command Context

        Note:
            This method normally will *never* return, as it is a prototype. It should continue indefinitely

        Returns:
             bool: True iff no failures occur, otherwise False

        """
        if context.get('foreground'):
            # set foreground if in context
            self.ioc.getLogger().foreground = True
        self.ioc.getLogger().trace("Kafka starting", trace=True)

        kafka_source = KafkaSource(self.ioc)
        try:
            kafka_source.run(context.get("config"))
        except KeyboardInterrupt:
            # graceful close for scanning
            self.ioc.getLogger().trace("Keyboard interrupt in scanner detected", trace=True)
            return True
        # ensure we clean up after ourselves
        if context.get('foreground'):
            self.ioc.getLogger().foreground = False
        return False



from tgt_grease.core import Logging, Configuration, ImportTool
from tgt_grease.core.Types import Command
import os
import sys


class GreaseRouter(object):
    """Main GREASE CLI Router

    This class handles routing CLI requests as well as starting the Daemon on Windows/POSIX systems

    Attributes:
        _config (Configuration): Main Configuration Object
        _logger (Logging): Main Logging Instance
        _importTool (ImportTool): Importer Tool Instance
        _exit_message (str): Exit Message

    """

    _config = Configuration(os.environ.get('GREASE_CONF', None))
    _logger = Logging(_config)
    _importTool = ImportTool(_logger)
    _exit_message = None

    def __init__(self):
        self._logger.trace("Router Startup", trace=True)

    def StartGREASE(self):
        """EntryPoint for CLI scripts for GREASE

        Returns:
            None: Void Method for GREASE

        """
        status = self.run()
        self.exit(status, self._exit_message)

    def run(self):
        """Route commands through GREASE

        Returns:
            int: Exit Code

        """
        # ensure at least a sub-command has been provided
        if len(sys.argv) > 1:
            cmd, context = self.get_arguments()
            if cmd:
                # Parse long args to command context
                if cmd.execute(context):
                    cmd.__del__()
                    del cmd
                    return 0
                else:
                    return 3
            else:
                self._exit_message = "Command not found"
                return 2
        else:
            self._logger.error("Sub-command not provided")
            self._exit_message = "Sub-command not provided to GREASE CLI"
            return 1

    def exit(self, code, message=None):
        """Exit program with exit code

        Args:
            code (int): Exit Code
            message (str): Exit message if any

        Returns:
            None: Will exit program

        """
        if message:
            self._logger.info("Message: [{0}]".format(message))
            if code != 0:
                print("ERROR: {0}".format(message))
            else:
                print(message)
        self._logger.debug("GREASE exit code: [{0}]".format(code), verbose=True)
        sys.exit(code)

    def get_arguments(self):
        """Parse CLI long arguments into dictionaries

        This expects arguments separated by space `--opt val`, colon `--opt:val`, or equal `--opt=val` signs

        Returns:
            object, dict: key->value pairs of arguments

        """
        i = 1
        context = {}
        other = []
        cmd = None
        while i < len(sys.argv):
            arg = str(sys.argv[i])
            if arg.startswith("--"):
                # Found long opt
                if len(arg.split("=")) > 1:
                    # was equal separated
                    context[arg.split("=")[0].strip("--")] = arg.split("=")[1]
                elif len(arg.split(":")) > 1:
                    # was colon separated
                    context[arg.split(":")[0].strip("--")] = arg.split(":")[1]
                else:
                    if len(sys.argv) < i + 1:
                        # we have a flag rather than an arg
                        context[arg.strip("--")] = True
                        i += 1
                    elif len(sys.argv) - 1 == i or sys.argv[i + 1].startswith("--"):
                        # we have a flag rather than an arg
                        context[arg.strip("--")] = True
                    elif sys.argv[i + 1].startswith("--"):
                        # we have a flag rather than an arg
                        context[arg.strip("--")] = True
                    else:
                        # space separated
                        possible_imp = self._importTool.load(sys.argv[i + 1])
                        if not isinstance(possible_imp, Command):
                            context[arg.strip("--")] = sys.argv[i + 1]
                        else:
                            cmd = possible_imp
                        i += 1
            else:
                possible_imp = self._importTool.load(str(sys.argv[i]))
                if isinstance(possible_imp, Command):
                    cmd = possible_imp
                else:
                    other.append(str(arg))
            i += 1
        context['grease_other_args'] = other
        return cmd, context



from unittest import TestCase
from tgt_grease.management.Model import BridgeCommand
from bson.objectid import ObjectId


class TestBridge(TestCase):

    def test_registration(self):
        b = BridgeCommand()
        self.assertTrue(b.action_register())

    def test_info(self):
        b = BridgeCommand()
        self.assertTrue(b.action_info(jobs=True, prototypeJobs=True))

    def test_info_with_oid(self):
        b = BridgeCommand()
        self.assertTrue(b.action_info(node=b.ioc.getConfig().NodeIdentity, jobs=True, prototypeJobs=True))

    def test_info_wih_bad_oid(self):
        b = BridgeCommand()
        self.assertFalse(b.action_info(node='32j45koHJO34523o', jobs=True, prototypeJobs=True))

    def test_assignment_operations(self):
        b = BridgeCommand()
        self.assertTrue(b.action_assign(prototype='scan'))
        self.assertTrue(b.action_assign(role='stage'))
        self.assertTrue(b.action_assign(prototype='detect', role='dev'))
        self.assertTrue(b.ioc.getCollection('JobServer').find({
            '_id': ObjectId(b.ioc.getConfig().NodeIdentity),
            'prototypes': 'scan'
        }).count())
        self.assertTrue(b.action_unassign(prototype='scan'))
        self.assertTrue(b.action_unassign(role='stage'))
        self.assertTrue(b.action_unassign(prototype='detect', role='dev'))

    def test_culling_operations(self):
        b = BridgeCommand()
        self.assertTrue(b.action_cull(b.ioc.getConfig().NodeIdentity))
        self.assertTrue(b.ioc.getCollection('JobServer').find({
            '_id': ObjectId(b.ioc.getConfig().NodeIdentity),
            'active': False
        }).count())
        self.assertTrue(b.action_activate(b.ioc.getConfig().NodeIdentity))
        self.assertTrue(b.ioc.getCollection('JobServer').find({
            '_id': ObjectId(b.ioc.getConfig().NodeIdentity),
            'active': True
        }).count())

    def test_node_validation(self):
        b = BridgeCommand()
        valid, server = b.valid_server()
        self.assertTrue(valid)
        valid, server = b.valid_server(b.ioc.getConfig().NodeIdentity)
        self.assertTrue(valid)

    def test_node_validation_failed(self):
        b = BridgeCommand()
        valid, server = b.valid_server('45u32890JKLdsfikojnf')
        self.assertFalse(valid)




from .Configuration import Configuration
from .Notifier import Notifications
from .Logging import Logging
from .Importer import ImportTool
from .Connectivity import Mongo
from .InversionOfControl import GreaseContainer
from . import Types



from tgt_grease.core.Types import Command
from tgt_grease.management.Model import BridgeCommand


class Bridge(Command):
    """CLI tool for cluster administration

    The command palate is listed here::

        Args:
            register
                register this node with a GREASE Cluster as provided in the configuration file
            info
                --node:<ObjectID>
                    !Optional! parameter to observe a remote node. Defaults to look at self
                --jobs
                    !Optional! if set will list jobs executed
                --pJobs
                    !Optional! include Prototype Jobs in list of jobs
            assign
                --prototype:<string>
                    !mandatory if assigning a prototype! prototype to assign
                    !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
                --role:<string>
                    !mandatory if assigning a role! role to assign
                    !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
                --node:<ObjectID>
                    !Optional! remote node to assign job to
            unassign
                --prototype:<string>
                    !mandatory if unassigning a prototype! prototype to unassign
                    !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
                --role:<string>
                    !mandatory if unassigning a role! role to unassign
                    !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
                --node:<ObjectID>
                    !Optional! remote node to unassign job to
            cull
                --node:<ObjectID>
                    !Optional! parameter to cull a remote node. Defaults to look at self
            activate
                --node:<ObjectID>
                    !Optional! parameter to activate a remote node. Defaults to look at self
            --foreground
                If set will print log messages to the commandline

    Note:
        This tool is ever evolving! If you need something more feel free to create an issue!

    Attributes:
        bridge (BridgeCommand): Model Instance

    """

    __author__ = "James E. Bell Jr."
    __version__ = "2.0.0"
    purpose = "Control node/cluster operations"
    help = """
    CLI for administrators to manage GREASE Clusters
    
    Args:
        register
            register this node with a GREASE Cluster as provided in the configuration file
        info
            --node:<ObjectID>
                !Optional! parameter to observe a remote node. Defaults to look at self
            --jobs
                !Optional! if set will list jobs executed
            --pJobs
                !Optional! include Prototype Jobs in list of jobs
        assign
            --prototype:<string>
                !mandatory if assigning a prototype! prototype to assign
                !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
            --role:<string>
                !mandatory if assigning a role! role to assign
                !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
            --node:<ObjectID>
                !Optional! remote node to assign job to
        unassign
            --prototype:<string>
                !mandatory if unassigning a prototype! prototype to unassign
                !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
            --role:<string>
                !mandatory if unassigning a role! role to unassign
                !NOTE! THIS MUST BE SEPARATED BY COLON OR EQUAL SIGN
            --node:<ObjectID>
                !Optional! remote node to unassign job to
        cull
            --node:<ObjectID>
                !Optional! parameter to cull a remote node. Defaults to look at self
        activate
            --node:<ObjectID>
                !Optional! parameter to activate a remote node. Defaults to look at self
        --foreground
            If set will print log messages to the commandline
    
    """

    def __init__(self):
        super(Bridge, self).__init__()
        self.bridge = BridgeCommand(self.ioc)

    def execute(self, context):
        """This method monitors the environment

        An [in]finite loop monitoring the cluster nodes for unhealthy ones

        Args:
            context (dict): context for the command to use

        Returns:
            bool: Command Success

        """
        retVal = False
        if context.get('foreground'):
            self.ioc.getLogger().foreground = True
        if 'register' in context.get('grease_other_args', []):
            retVal = self.bridge.action_register()
        elif 'info' in context.get('grease_other_args', []):
            retVal = self.bridge.action_info(context.get('node'), context.get('jobs'), context.get('pJobs'))
        elif 'assign' in context.get('grease_other_args', []):
            retVal = self.bridge.action_assign(context.get('prototype'), context.get('role'), context.get('node'))
        elif 'unassign' in context.get('grease_other_args', []):
            retVal = self.bridge.action_unassign(context.get('prototype'), context.get('role'), context.get('node'))
        elif 'cull' in context.get('grease_other_args', []):
            retVal = self.bridge.action_cull(context.get('node'))
        elif 'activate' in context.get('grease_other_args', []):
            retVal = self.bridge.action_activate(context.get('node'))
        else:
            print("Sub-command Not Found! Here is the help information:")
            print(self.help)
            retVal = False
        if context.get('foreground'):
            self.ioc.getLogger().foreground = False
        return retVal



from unittest import TestCase
from unittest.case import SkipTest
from tgt_grease.core import GreaseContainer
from tgt_grease.enterprise.Model import Detect
import pkg_resources
import os
import json


class AutomationTest(TestCase):
    """Automation Test Class

    Version II of GREASE was all about proving stability. Automation testing is critically important to ensure
    reliability during fault isolation. This class is an abstract class your tests can implement to ensure they
    will perform exactly as you expect in production.

    Make sure you set the **configuration** class attribute to ensure your configuration is tested, the **mock_data**
    class attribute with your mock data dictionary you expect to be sourced in production, and the **expected_data**
    with what you expect detection to find from your mocked source data. Then implement the **test_command** method to
    write standard unittests around your automation. The Platform will test your configuration for you, and execute
    **test_command** with `python setup.py test` is executed.

    Attributes:
        configuration (str|dict): Configuration to load for this test
        mock_data (dict): String Key -> Int/Float/String Value pair to mock source data
        expected_data (dict): data you expect context for your command to look like
        enabled (bool): set to true to enable your test to run

    Here is an example::

        class TestAutomationTest(AutomationTest):

            def __init__(self, *args, **kwargs):
                AutomationTest.__init__(self, *args, **kwargs)
                self.configuration = "mongo://test_automation_test"
                self.mock_data = {'ver': 'var'}
                self.expected_data = {'ver': ['var']}
                self.enabled = True

            def test_command(self):
                myCommand = myCommand()
                self.assertTrue(myCommand.execute({'hostname': 'localhost'}))

    This is a pretty basic example but it will help you get started automatically testing your automation!

    Note:
        **YOU MUST SET THE PROPERTY `ENABLED` TO BOOLEAN TRUE IN ORDER FOR YOUR TEST TO BE PICKED UP**
    Note:
        To use a static configuration set `configuration` to a dictionary
    Note:
        To use a MongoDB configuration for a test prefix your configuration's name with mongo://
    Note:
        To use a package configuration for a test prefix your configuration's name with pkg://
    Note:
        to use a filesystem configuration for a test prefix your configuration's path with fs://

    """

    def __init__(self, *args, **kwargs):
        TestCase.__init__(self, *args, **kwargs)
        self.configuration = None
        self.enabled = False
        self.mock_data = {}
        self.expected_data = {}
        self.ioc = GreaseContainer()
        self.detect = Detect(self.ioc)

    def test_configuration(self):
        """Configuration Test

        This method tests your configuration and validates that detection will return as you expect

        """
        if not self.enabled:
            raise SkipTest
        self.assertTrue(self.configuration, "Ensure configuration is not empty")
        self.assertIsInstance(self.configuration, str, "Ensure configuration is type string")
        self.assertTrue(self.mock_data, "Ensure mock_data is not empty")
        self.assertIsInstance(self.mock_data, dict, "Ensure mock_data is type dict")
        self.assertTrue(self.expected_data, "Ensure expected_data is not empty")
        self.assertIsInstance(self.expected_data, dict, "Ensure expected_data is type dict")
        config = None
        if str(self.configuration).startswith("mongo://"):
            config = self.ioc.getCollection('Configuration').find_one({
                'name': str(self.configuration).split("://")[1],
                'active': True,
                "type": "prototype_config"
            })
            self.assertTrue(config, "Ensuring MongoDB has configuration")
            config = dict(config)
        elif str(self.configuration).startswith("pkg://"):
            if os.path.isfile(pkg_resources.resource_filename('tgt_grease', str(self.configuration).split("://")[1])):
                with open(pkg_resources.resource_filename('tgt_grease', str(self.configuration).split("://")[1]), 'rb') as fil:
                    config = json.loads(fil.read())
                    self.assertIsInstance(config, dict, "Ensuring Valid JSON")
            else:
                self.assertTrue(
                    False,
                    "Failed to load [{0}] from tgt_grease pkg".format(str(self.configuration).split("://")[1])
                )
        elif str(self.configuration).startswith("fs://"):
            if os.path.isfile(str(self.configuration).split("://")[1]):
                with open(str(self.configuration).split("://")[1], 'rb') as fil:
                    config = json.loads(fil.read())
                    self.assertIsInstance(config, dict, "Ensuring Valid JSON")
            else:
                self.assertTrue(
                    False,
                    "Failed to load [{0}] from filesystem".format(str(self.configuration).split("://")[1])
                )
        else:
            self.assertTrue(False, "Failed to load configuration::Invalid Configuration Location Type")
        self.assertTrue(config, "Ensuring config is not boolean equatable to False")
        result, context = self.detect.detection(self.mock_data, config)
        self.assertTrue(result, "Detection Results")
        self.assertDictEqual(context, self.expected_data, "validating context expected")

    def test_command(self):
        """This method is for **you** to fill out to test your command

        Note:
            The more tests the better! Make sure to add as many tests as you need to ensure your automation is always successful

        """
        if not self.enabled:
            raise SkipTest



from tgt_grease.core import GreaseContainer, ImportTool
from .Configuration import PrototypeConfig
from .CentralScheduling import Scheduling
from bson.objectid import ObjectId
import pymongo
import datetime


class Scheduler(object):
    """Job Scheduler Model

    This model will attempt to schedule a job for execution

    Attributes:
        ioc (GreaseContainer): IOC for scanning
        impTool (ImportTool): Import Utility Instance
        conf (PrototypeConfig): Prototype configuration tool
        scheduler (Scheduling): Prototype Scheduling Service Instance

    """

    def __init__(self, ioc=None):
        if ioc and isinstance(ioc, GreaseContainer):
            self.ioc = ioc
        else:
            self.ioc = GreaseContainer()
        self.impTool = ImportTool(self.ioc.getLogger())
        self.ioc.ensureRegistration()
        self.conf = PrototypeConfig(self.ioc)
        self.scheduler = Scheduling(self.ioc)

    def scheduleExecution(self):
        """Schedules the oldest successfully detected source to execution

        Returns:
            bool: True if detection is successful else false

        """
        source = self.getDetectedSource()
        if source:
            self.ioc.getLogger().trace("Attempting schedule of source", trace=True)
            self.ioc.getCollection('SourceData').update_one(
                {'_id': ObjectId(source.get('_id'))},
                {
                    '$set': {
                        'grease_data.scheduling.start': datetime.datetime.utcnow()
                    }
                }
            )
            if self.schedule(source):
                self.ioc.getLogger().trace("Scheduling [{0}] was successful".format(source['_id']), trace=True)
                self.ioc.getCollection('SourceData').update_one(
                    {'_id': ObjectId(source.get('_id'))},
                    {
                        '$set': {
                            'grease_data.scheduling.end': datetime.datetime.utcnow()
                        }
                    }
                )
                return True
            else:
                self.ioc.getLogger().error(
                    "Failed to schedule [{0}] for execution".format(source['_id']),
                    trace=True,
                    notify=False
                )
                self.ioc.getCollection('SourceData').update_one(
                    {'_id': ObjectId(source.get('_id'))},
                    {
                        '$set': {
                            'grease_data.scheduling.start': None,
                            'grease_data.scheduling.end': None
                        }
                    }
                )
                return False
        else:
            self.ioc.getLogger().trace("No sources detected for this node at this time", trace=True)
            return True

    def getDetectedSource(self):
        """Gets the oldest successfully detected source

        Returns:
            dict: Object from MongoDB

        """
        return self.ioc.getCollection('SourceData').find_one(
            {
                    'grease_data.scheduling.server': ObjectId(self.ioc.getConfig().NodeIdentity),
                    'grease_data.scheduling.start': None,
                    'grease_data.scheduling.end': None
            },
            sort=[('grease_data.createTime', pymongo.DESCENDING)]
        )

    def schedule(self, source):
        """Schedules source for execution

        Returns:
            bool: If scheduling was successful or not

        """
        if isinstance(source['configuration'], bytes):
            config = self.conf.get_config(source['configuration'].decode())
        else:
            config = self.conf.get_config(source['configuration'])
        if not config:
            self.ioc.getLogger().error("Failed to load configuration for source [{0}]".format(source['_id']))
            return False
        server, jobs = self.scheduler.determineExecutionServer(config.get('exe_env', 'general'))
        if not server:
            self.ioc.getLogger().error(
                "Failed to find an Execution Node for environment [{0}]".format(config.get('exe_env', 'general'))
            )
            return False
        self.ioc.getCollection('SourceData').update_one(
            {'_id': ObjectId(source['_id'])},
            {
                '$set': {
                    'grease_data.execution.server': ObjectId(server),
                    'grease_data.execution.assignmentTime': datetime.datetime.utcnow(),
                }
            }
        )
        self.ioc.getCollection('JobServer').update_one(
            {'_id': ObjectId(server)},
            {
                '$set': {
                    'jobs': jobs + 1
                }
            }
        )
        return True



from unittest import TestCase
from tgt_grease.core import GreaseContainer
from tgt_grease.core.Types import Command
from tgt_grease.router.Commands.Daemon import DaemonProcess
from tgt_grease.enterprise.Model import Deduplication, PrototypeConfig
from bson import ObjectId
from tgt_grease.core import Configuration
import json
import datetime
import time


class TestProtoType(Command):
    def __init__(self):
        super(TestProtoType, self).__init__()

    def execute(self, context):
        if self.ioc.getCollection('TestProtoType').find({'runs': {'$exists': True}}).count() <= 10:
            for i in range(0, 10):
                self.ioc.getCollection('TestProtoType').insert_one({'runs': i})
        return True


class TestRegistration(TestCase):
    def test_registration(self):
        ioc = GreaseContainer()
        cmd = DaemonProcess(ioc)
        self.assertTrue(cmd.register())
        collection = ioc.getMongo().Client().get_database('grease').get_collection('JobServer')
        self.assertTrue(collection.find({}).count())
        del collection
        del cmd
        del ioc

    def test_job_execution(self):
        ioc = GreaseContainer()
        cmd = DaemonProcess(ioc)
        proto = PrototypeConfig(ioc)
        ioc.getCollection('Configuration').insert_one(
            {
                'active': True,
                'type': 'prototype_config',
                "name": "exe_test",
                "job": "help",
                "exe_env": "general",
                "source": "url_source",
                "logic": {
                    "Regex": [
                        {
                            "field": "url",
                            "pattern": ".*",
                            'variable': True,
                            'variable_name': 'url'
                        }
                    ],
                    'Range': [
                        {
                            'field': 'status_code',
                            'min': 199,
                            'max': 201
                        }
                    ]
                },
                'constants': {
                    'test': 'ver'
                }
            }
        )
        proto.load(reloadConf=True)
        jobid = ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(ioc.getConfig().NodeIdentity),
                            'start': datetime.datetime.utcnow(),
                            'end': datetime.datetime.utcnow(),
                            'detection': {}
                        },
                        'scheduling': {
                            'server': ObjectId(ioc.getConfig().NodeIdentity),
                            'start': datetime.datetime.utcnow(),
                            'end': datetime.datetime.utcnow(),
                        },
                        'execution': {
                            'server': ObjectId(ioc.getConfig().NodeIdentity),
                            'assignmentTime': datetime.datetime.utcnow(),
                            'completeTime': None,
                            'returnData': {},
                            'executionSuccess': False,
                            'commandSuccess': False,
                            'failures': 0
                        }
                    },
                    'source': 'dev',
                    'configuration': 'exe_test',
                    'data': {},
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
                }).inserted_id
        # Run for a bit
        self.assertTrue(cmd.server())
        self.assertTrue(cmd.drain_jobs(ioc.getCollection('SourceData')))
        result = ioc.getCollection('SourceData').find_one({'_id': ObjectId(jobid)})
        self.assertTrue(result)
        self.assertTrue(result.get('grease_data').get('execution').get('executionSuccess'))
        self.assertTrue(result.get('grease_data').get('execution').get('commandSuccess'))
        ioc.getCollection('SourceData').drop()
        ioc.getCollection('Configuration').drop()

    def test_prototype_execution(self):
        ioc = GreaseContainer()
        cmd = DaemonProcess(ioc)
        # add search path
        fil = open(ioc.getConfig().greaseConfigFile, 'r')
        data = json.loads(fil.read())
        fil.close()
        fil = open(ioc.getConfig().greaseConfigFile, 'w')
        data['Import']['searchPath'].append('tgt_grease.router.Commands.tests')
        fil.write(json.dumps(data, sort_keys=True, indent=4))
        fil.close()
        Configuration.ReloadConfig()
        # Update Node to run it
        ioc.getCollection('JobServer')\
            .update_one(
            {'_id': ObjectId(ioc.getConfig().NodeIdentity)},
            {
                '$set': {
                    'prototypes': ['TestProtoType']
                }
            }
        )
        # Sleeps are because mongo in Travis is slow sometimes to persist data
        time.sleep(1.5)
        self.assertTrue(cmd.server())
        self.assertTrue(cmd.drain_jobs(ioc.getCollection('JobQueue')))
        # ensure jobs drain out
        time.sleep(1.5)
        self.assertEqual(
            ioc.getCollection('TestProtoType').find({'runs': {'$exists': True}}).count(),
            10
        )
        # clean up
        fil = open(ioc.getConfig().greaseConfigFile, 'r')
        data = json.loads(fil.read())
        fil.close()
        # remove collection
        ioc.getCollection('TestProtoType').drop()
        # pop search path
        trash = data['Import']['searchPath'].pop()
        # close out
        fil = open(ioc.getConfig().greaseConfigFile, 'w')
        fil.write(json.dumps(data, sort_keys=True, indent=4))
        fil.close()
        ioc.getCollection('JobServer') \
            .update_one(
            {'_id': ObjectId(ioc.getConfig().NodeIdentity)},
            {
                '$set': {
                    'prototypes': []
                }
            }
        )



from abc import ABCMeta, abstractmethod
from tgt_grease.core import Logging, GreaseContainer
from datetime import datetime
import sys
import os
import traceback


class Command(object):
    """Abstract class for commands in GREASE

    Attributes:
        __metaclass__ (ABCMeta): Metadata class object
        purpose (str): The purpose of the command
        help (str): Help string for the command line
        __author__ (str): Authorship string
        __version__ (str): Command Version
        os_needed (str): If a specific OS is needed then set this
        ioc (GreaseContainer): IOC container for access to system resources
        variable_storage (pymongo.collection): collection object for command

    """

    ###
    # Command Metadata information
    ###
    purpose = "Default"
    help = """
    No Help Information Provided
    """
    __author__ = "Jimmy The Programmer"
    __version__ = "1.0.0"
    os_needed = None
    __metaclass__ = ABCMeta

    def __init__(self, Logger=None):
        if Logging and isinstance(Logger, Logging):
            self.ioc = GreaseContainer(Logger)
        else:
            self.ioc = GreaseContainer()
        self.variable_storage = self.ioc.getMongo()\
            .Client()\
            .get_database(self.ioc.getConfig().get('Connectivity', 'MongoDB').get('db', 'grease'))\
            .get_collection(self.__class__.__name__)
        self.start_time = datetime.utcnow()
        self.exec_data = {'execVal': False, 'retVal': False, 'data': {}}

    def getExecVal(self):
        """Get the execution attempt success

        Returns:
            bool: If the command executed without exception

        """
        return self.exec_data.get('execVal', False)

    def getRetVal(self):
        """Get the execution boolean return state

        Returns:
            bool: the boolean return value of execute

        """
        return self.exec_data.get('retVal', False)

    def getData(self):
        """Get any data the execute method wanted to put into telemetry

        Returns:
            dict: The Key/Value pairs from the execute method execution

        """
        return self.exec_data.get('data', {})

    def setData(self, Key, Data):
        """Put Data into the data object to be inserted into telemetry

        Args:
            Key (str): Key for the data to be stored
            Data (object): JSON-able object to store

        Returns:
            None: Void Method to put data

        """
        self.exec_data['data'][Key] = Data

    def __del__(self):
        # close mongo connection
        self.ioc.getMongo().Close()

    def safe_execute(self, context=None):
        """Attempt execution and prevent MOST exceptions

        Args:
            context (dict): context for the command to use

        Returns:
            None: Void method to attempt exceptions

        """
        if not context:
            context = {}
        try:
            try:
                self.exec_data['execVal'] = True
                self.exec_data['retVal'] = bool(self.execute(context))
            except BaseException:
                self.exec_data['execVal'] = False
                exc_type, exc_obj, exc_tb = sys.exc_info()
                tb = traceback.format_exception(exc_type, exc_obj, exc_tb)
                self.ioc.getLogger().error(
                    "Failed to execute [{0}] execute got exception!".format(self.__class__.__name__),
                    additional={
                        'file': os.path.split(str(str(tb[2]).split('"')[1]))[1],
                        'type': str(exc_type),
                        'line': str(str(tb[2]).split(",")[1]).split(' ')[2]
                    }
                )
            except:
                self.ioc.getLogger().error(
                    "Failed to execute [{0}] execute got exception!".format(self.__class__.__name__),
                )
        except:
            self.ioc.getLogger().error(
                "Failed to execute [{0}] execute major exception".format(self.__class__.__name__),
            )

    @abstractmethod
    def execute(self, context):
        """Base Execute Method

        This method should *always* be overridden in child classes. This is the code that will run when your command
        is called. If this method is not implemented then the class will fail loading.

        Args:
            context (dict): context for the command to use

        Returns:
            bool: Command Success

        """
        pass



from tgt_grease.core import GreaseContainer
from tgt_grease.core.Types import Command
from bson.objectid import ObjectId
from bson.errors import InvalidId
from tgt_grease.core import ImportTool
from tgt_grease.management.Model import NodeMonitoring
import datetime


class BridgeCommand(object):
    """Methods for Cluster Administration

    Attributes:
        imp (ImportTool): Import Tool Instance
        monitor (NodeMonitoring): Node Monitoring Model Instance

    """

    def __init__(self, ioc=None):
        if isinstance(ioc, GreaseContainer):
            self.ioc = ioc
        else:
            self.ioc = GreaseContainer()
        self.imp = ImportTool(self.ioc.getLogger())
        self.monitor = NodeMonitoring(self.ioc)

    def action_register(self):
        """Ensures Registration of server

        Returns:
            bool: Registration status

        """
        self.ioc.getLogger().debug("Registration Requested")
        if self.ioc.ensureRegistration():
            print("Registration Complete!")
            self.ioc.getLogger().info("Registration Completed Successfully")
            return True
        else:
            print("Registration Failed!")
            self.ioc.getLogger().info("Registration Failed")
            return False

    def action_info(self, node=None, jobs=None, prototypeJobs=None):
        """Gets Node Information

        Args:
            node (str): MongoDB Object ID to get information about
            jobs (bool): If true then will retrieve jobs executed by this node
            prototypeJobs (bool): If true then prototype jobs will be printed as well

        Note:
            provide a node argument via the CLI --node=4390qwr2fvdew458239
        Note:
            provide a jobs argument via teh CLI --jobs
        Note:
            provide a prototype jobs argument via teh CLI --pJobs

        Returns:
            bool: If Info was found

        """
        if not self.ioc.ensureRegistration():
            self.ioc.getLogger().error("Server not registered with MongoDB")
            print("Unregistered servers cannot talk to the cluster")
            return False
        valid, serverId = self.valid_server(node)
        if not valid:
                print("Invalid ObjectID")
                return False
        server = self.ioc.getCollection('JobServer').find_one({'_id': ObjectId(str(serverId))})
        if server:
            server = dict(server)
            print("""
<<<<<<<<<<<<<< SERVER: {0} >>>>>>>>>>>>>>
Activation State: {1} Date: {2}
Jobs: {3}
Operating System: {4}
Prototypes: {5}
Execution Roles: {6}
            """.format(
                server.get('_id'),
                server.get('active'),
                server.get('activationTime'),
                server.get('jobs'),
                server.get('os'),
                server.get('prototypes'),
                server.get('roles'))
            )
            if jobs and prototypeJobs:
                print("======================= SOURCING =======================")
                for job in self.ioc.getCollection('SourceData').find({'grease_data.sourcing.server': ObjectId(serverId)}):
                    print("""
-------------------------------
Job: {0}
-------------------------------
                    """, job['_id'])
            if jobs and prototypeJobs:
                print("======================= DETECTION =======================")
                for job in self.ioc.getCollection('SourceData').find({'grease_data.detection.server': ObjectId(serverId)}):
                    print("""
-------------------------------
Job: {0}
Start Time: {1}
End Time: {2}
Context: {3}
-------------------------------
                    """.format(
                        job['_id'],
                        job['grease_data']['detection']['start'],
                        job['grease_data']['detection']['end'],
                        job['grease_data']['detection']['detection'])
                    )
            if jobs and prototypeJobs:
                print("======================= SCHEDULING =======================")
                for job in self.ioc.getCollection('SourceData').find({'grease_data.scheduling.server': ObjectId(serverId)}):
                    print("""
-------------------------------
Job: {0}
Start Time: {1}
End Time: {2}
-------------------------------
                    """.format(
                        job['_id'],
                        job['grease_data']['scheduling']['start'],
                        job['grease_data']['scheduling']['end'])
                    )
            if jobs:
                print("======================= EXECUTION =======================")
                for job in self.ioc.getCollection('SourceData').find({'grease_data.execution.server': ObjectId(serverId)}):
                    print("""
-------------------------------
Job: {0}
Assignment Time: {1}
Completed Time: {2}
Execution Success: {3}
Command Success: {4}
Failures: {5}
Return Data: {6}
-------------------------------
                    """.format(
                        job['_id'],
                        job['grease_data']['execution']['assignmentTime'],
                        job['grease_data']['execution']['completeTime'],
                        job['grease_data']['execution']['executionSuccess'],
                        job['grease_data']['execution']['commandSuccess'],
                        job['grease_data']['execution']['failures'],
                        job['grease_data']['execution']['returnData'])
                    )
            return True
        else:
            print("Unable to locate server")
            self.ioc.getLogger().error("Unable to load [{0}] server for information".format(serverId))
            return False

    def action_assign(self, prototype=None, role=None, node=None):
        """Assign prototypes/roles to a node either local or remote

        Args:
            prototype (str): Prototype Job to assign
            role (str): Role to assign
            node (str): MongoDB ObjectId of node to assign to, if not provided will default to the local node

        Returns:
            bool: If successful true else false

        """
        assigned = False
        if prototype:
            job = self.imp.load(str(prototype))
            if not job or not isinstance(job, Command):
                print("Cannot find prototype [{0}] to assign check search path!".format(prototype))
                self.ioc.getLogger().error("Cannot find prototype [{0}] to assign check search path!".format(prototype))
                return False
            # Cleanup job
            job.__del__()
            del job
            valid, serverId = self.valid_server(node)
            if not valid:
                    print("Invalid ObjectID")
                    return False
            updated = self.ioc.getCollection('JobServer').update_one(
                {'_id': ObjectId(serverId)},
                {
                    '$push': {
                        'prototypes': prototype
                    }
                }
            ).acknowledged
            if updated:
                print("Prototype Assigned")
                self.ioc.getLogger().info("Prototype [{0}] assigned to server [{1}]".format(prototype, serverId))
                assigned = True
            else:
                print("Prototype Assignment Failed!")
                self.ioc.getLogger().info("Prototype [{0}] assignment failed to server [{1}]".format(prototype, serverId))
                return False
        if role:
            valid, serverId = self.valid_server(node)
            if not valid:
                    print("Invalid ObjectID")
                    return False
            updated = self.ioc.getCollection('JobServer').update_one(
                {'_id': ObjectId(serverId)},
                {
                    '$push': {
                        'roles': role
                    }
                }
            ).acknowledged
            if updated:
                print("Role Assigned")
                self.ioc.getLogger().info("Role [{0}] assigned to server [{1}]".format(prototype, serverId))
                assigned = True
            else:
                print("Role Assignment Failed!")
                self.ioc.getLogger().info(
                    "Role [{0}] assignment failed to server [{1}]".format(prototype, serverId))
                return False
        if not assigned:
            print("Assignment failed, please check logs for details")
        return assigned

    def action_unassign(self, prototype=None, role=None, node=None):
        """Unassign prototypes to a node either local or remote

        Args:
            prototype (str): Prototype Job to unassign
            role (str): Role to unassign
            node (str): MongoDB ObjectId of node to unassign to, if not provided will default to the local node

        Returns:
            bool: If successful true else false

        """
        unassigned = False
        if prototype:
            job = self.imp.load(str(prototype))
            if not job or not isinstance(job, Command):
                print("Cannot find prototype [{0}] to unassign check search path!".format(prototype))
                self.ioc.getLogger().error("Cannot find prototype [{0}] to unassign check search path!".format(prototype))
                return False
            # Cleanup job
            job.__del__()
            del job
            valid, serverId = self.valid_server(node)
            if not valid:
                    print("Invalid ObjectID")
                    return False
            updated = self.ioc.getCollection('JobServer').update_one(
                {'_id': ObjectId(serverId)},
                {
                    '$pull': {
                        'prototypes': prototype
                    }
                }
            ).acknowledged
            if updated:
                print("Prototype Assignment Removed")
                self.ioc.getLogger().info("Prototype [{0}] unassigned from server [{1}]".format(prototype, serverId))
                unassigned = True
            else:
                print("Prototype Unassignment Failed!")
                self.ioc.getLogger().info("Prototype [{0}] unassignment failed from server [{1}]".format(prototype, serverId))
                return False
        if role:
            valid, serverId = self.valid_server(node)
            if not valid:
                    print("Invalid ObjectID")
                    return False
            updated = self.ioc.getCollection('JobServer').update_one(
                {'_id': ObjectId(serverId)},
                {
                    '$pull': {
                        'roles': role
                    }
                }
            ).acknowledged
            if updated:
                print("Role Removed")
                self.ioc.getLogger().info("Role [{0}] removed to server [{1}]".format(prototype, serverId))
                unassigned = True
            else:
                print("Role Removal Failed!")
                self.ioc.getLogger().info(
                    "Role [{0}] removal failed to server [{1}]".format(prototype, serverId))
                return False
        if not unassigned:
            print("Unassignment failed, please check logs for details")
        return unassigned

    def action_cull(self, node=None):
        """Culls a server from the active cluster

        Args:
            node (str): MongoDB ObjectId to cull; defaults to local node

        """
        if not self.ioc.ensureRegistration():
            self.ioc.getLogger().error("Server not registered with MongoDB")
            print("Unregistered servers cannot talk to the cluster")
            return False
        valid, serverId = self.valid_server(node)
        if not valid:
                print("Invalid ObjectID")
                return False
        if not self.monitor.deactivateServer(serverId):
            self.ioc.getLogger().error(
                "Failed deactivating server [{0}]".format(serverId)
            )
            print("Failed deactivating server [{0}]".format(serverId))
            return False
        self.ioc.getLogger().warning(
            "Server [{0}] preparing to reallocate detect jobs".format(serverId)
        )
        if not self.monitor.rescheduleDetectJobs(serverId):
            self.ioc.getLogger().error(
                "Failed rescheduling detect jobs [{0}]".format(serverId)
            )
            print("Failed rescheduling detect jobs [{0}]".format(serverId))
            return False
        self.ioc.getLogger().warning(
            "Server [{0}] preparing to reallocate schedule jobs".format(serverId)
        )
        if not self.monitor.rescheduleScheduleJobs(serverId):
            self.ioc.getLogger().error(
                "Failed rescheduling detect jobs [{0}]".format(serverId)
            )
            print("Failed rescheduling detect jobs [{0}]".format(serverId))
            return False
        self.ioc.getLogger().warning(
            "Server [{0}] preparing to reallocate jobs".format(serverId)
        )
        if not self.monitor.rescheduleJobs(serverId):
            self.ioc.getLogger().error(
                "Failed rescheduling detect jobs [{0}]".format(serverId)
            )
            print("Failed rescheduling detect jobs [{0}]".format(serverId))
            return False
        print("Server Deactivated")
        return True

    def action_activate(self, node=None):
        """activates server in cluster

        Args:
            node (str): MongoDB ObjectId to activate; defaults to local node

        Returns:
            bool: If activation is successful

        """
        if not self.ioc.ensureRegistration():
            self.ioc.getLogger().error("Server not registered with MongoDB")
            print("Unregistered servers cannot talk to the cluster")
            return False
        valid, serverId = self.valid_server(node)
        if not valid:
                print("Invalid ObjectID")
                return False
        if self.ioc.getCollection('JobServer').update_one(
                {'_id': ObjectId(serverId)},
                {
                    '$set': {
                        'active': True,
                        'activationTime': datetime.datetime.utcnow()
                    }
                }
        ).modified_count < 1:
            self.ioc.getLogger().warning("Server [{0}] failed to be activated".format(serverId))
            return False
        else:
            self.ioc.getLogger().warning("Server [{0}] activated".format(serverId))
            return True

    def valid_server(self, node=None):
        """Validates node is in the MongoDB instance connected to

        Args:
            node (str): MongoDB Object ID to validate; defaults to local node

        Returns:
            tuple: first element is boolean if valid second is objectId as string

        """
        if node:
            try:
                server = self.ioc.getCollection('JobServer').find_one({'_id': ObjectId(str(node))})
            except InvalidId:
                self.ioc.getLogger().error("Invalid ObjectID passed to bridge info [{0}]".format(node))
                return False, ""
            if server:
                return True, dict(server).get('_id')
            else:
                self.ioc.getLogger().error("Failed to find server [{0}] in the database".format(node))
                return False, ""
        else:
            return True, self.ioc.getConfig().NodeIdentity



import pymongo
from tgt_grease.core import Configuration


class Mongo(object):
    """MongoDB Connection Class

    Attributes:
        _client (pymongo.MongoClient): The actual PyMongo Connection
        _config (Configuration): Configuration Object

    """

    _client = None
    _config = None

    def __init__(self, Config=None):
        if Config and isinstance(Config, Configuration):
            self._config = Config
        else:
            self._config = Configuration()
        self._client = self._generate_client()

    def Client(self):
        """get the connection client

        Returns:
            pymongo.MongoClient: Returns the mongoDB connection client

        """
        return self._client

    def Close(self):
        """Close PyMongo Connection

        Returns:
            None: Void Method to close connection

        """
        self._client.close()

    def _generate_client(self):
        """Creates a PyMongo Client

        Returns:
            pymongo.MongoClient: Mongo Connection

        """
        mongoConf = self._config.get('Connectivity', 'MongoDB')  # type: dict
        if mongoConf.get('username') and mongoConf.get('password'):
            return pymongo.MongoClient(
                "mongodb://{0}:{1}@{2}:{3}/{4}".format(
                    mongoConf.get('username', ''),
                    mongoConf.get('password', ''),
                    mongoConf.get('host', 'localhost'),
                    mongoConf.get('port', 27017),
                    mongoConf.get('db', 'grease')
                ),
                w=1
            )
        else:
            return pymongo.MongoClient(
                host=mongoConf.get('host', 'localhost'),
                port=mongoConf.get('port', 27017),
                w=1
            )



from unittest import TestCase
from tgt_grease.enterprise.Sources import sql_source
from tgt_grease.core import Configuration
import json
import os
import psycopg2
import datetime
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT


class TestSQLSource(TestCase):

    def test_type(self):
        inst = sql_source()
        self.assertTrue(isinstance(inst, sql_source))

    def __ensure_schema(self):
        # Helper function
        if not os.environ.get('GREASE_TEST_DSN'):
            os.environ['GREASE_TEST_DSN'] = "host=localhost user=postgres"
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            with conn.cursor() as cursor:
                # just to make sure nothing exists
                try:
                    cursor.execute("""
                      SELECT  
                        pg_terminate_backend(pid) 
                        FROM pg_stat_activity 
                        WHERE datname='test_data'
                    """)
                    cursor.execute("""
                        DROP DATABASE test_data;
                    """)
                except:
                    print("Exception occurred during ensure schema... most of the time this is fine")
                try:
                    cursor.execute("""
                        CREATE DATABASE test_data;
                    """)
                except psycopg2.ProgrammingError as e:
                    print("Schema Exists: {0}".format(e.pgerror))
        if not os.environ.get('GREASE_TEST_DSN_ORIGINAL'):
            os.environ['GREASE_TEST_DSN_ORIGINAL'] = os.environ.get('GREASE_TEST_DSN')
        os.environ['GREASE_TEST_DSN'] = os.environ['GREASE_TEST_DSN'] + " dbname=test_data"

    def __cleanup_schema(self):
        with psycopg2.connect(os.environ['GREASE_TEST_DSN_ORIGINAL']) as conn:
            conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            with conn.cursor() as cursor:
                cursor.execute("""
                  SELECT  
                    pg_terminate_backend(pid) 
                    FROM pg_stat_activity 
                    WHERE datname='test_data'
                """)
                try:
                    cursor.execute("""
                        DROP DATABASE test_data;
                    """)
                except psycopg2.ProgrammingError as e:
                    print("Schema Does Not Exist: {0}".format(e.pgerror))
        os.environ['GREASE_TEST_DSN'] = os.environ['GREASE_TEST_DSN_ORIGINAL']

    def test_sql_parser_mock(self):
        source = sql_source()
        conf = Configuration()
        mock = {
            'id': 1,
            'name_fs': 'sally',
            'name_ls': 'sue'

        }
        fil = open(conf.greaseDir + 'etc' + conf.fs_sep + 'test.mock.sql.json', 'w')
        fil.write(json.dumps(mock))
        fil.close()
        mockData = source.mock_data({})
        self.assertEqual(len(mockData), 1)
        self.assertEqual(mock.get('id'), 1)
        self.assertEqual(mock.get('name_fs'), mockData[0].get('name_fs'))
        self.assertEqual(mock.get('name_ls'), mockData[0].get('name_ls'))
        os.remove(conf.greaseDir + 'etc' + conf.fs_sep + 'test.mock.sql.json')

    def test_sql_parser(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 1)
                self.assertIsInstance(data[0], dict)
                self.assertEqual(data[0].get('id'), 1)
                self.assertEqual(data[0].get('name_fs'), 'sally')
                self.assertEqual(data[0].get('name_ls'), 'sue')
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()

    def test_sql_parser_bad_type(self):
        source = sql_source()
        self.assertFalse(source.parse_source({
            'name': 'example_source',
            'job': 'example_job',
            'exe_env': 'general',
            'source': 'sql_source',
            'type': 'mssql',
            'dsn': 'GREASE_TEST_DSN',
            'query': 'select * from test_data;',
            'logic': {}
        }))
        data = source.get_data()
        self.assertTrue(isinstance(data, list))
        self.assertEqual(len(data), 0)

    def test_sql_parser_hour_good(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'hour': datetime.datetime.utcnow().hour,
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 1)
                self.assertIsInstance(data[0], dict)
                self.assertEqual(data[0].get('id'), 1)
                self.assertEqual(data[0].get('name_fs'), 'sally')
                self.assertEqual(data[0].get('name_ls'), 'sue')
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()

    def test_sql_parser_minute_good(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'minute': datetime.datetime.utcnow().minute,
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 1)
                self.assertIsInstance(data[0], dict)
                self.assertEqual(data[0].get('id'), 1)
                self.assertEqual(data[0].get('name_fs'), 'sally')
                self.assertEqual(data[0].get('name_ls'), 'sue')
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()

    def test_sql_parser_hour_and_minute_good(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'hour': datetime.datetime.utcnow().hour,
                    'minute': datetime.datetime.utcnow().minute,
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 1)
                self.assertIsInstance(data[0], dict)
                self.assertEqual(data[0].get('id'), 1)
                self.assertEqual(data[0].get('name_fs'), 'sally')
                self.assertEqual(data[0].get('name_ls'), 'sue')
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()

    def test_sql_parser_hour_bad(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'hour': (datetime.datetime.utcnow() + datetime.timedelta(hours=6)).hour,
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 0)
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()

    def test_sql_parser_minute_bad(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'minute': (datetime.datetime.utcnow() + datetime.timedelta(minutes=10)).minute,
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 0)
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()

    def test_sql_parser_hour_and_minute_bad(self):
        self.__ensure_schema()
        with psycopg2.connect(os.environ['GREASE_TEST_DSN']) as conn:
            with conn.cursor() as cursor:
                source = sql_source()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS 
                      test_data
                    (
                        id SERIAL PRIMARY KEY NOT NULL,
                        name_fs VARCHAR,
                        name_ls VARCHAR
                    );
                """)
                conn.commit()
                cursor.execute("""
                    INSERT INTO
                      test_data
                    (name_fs, name_ls)
                    VALUES 
                    ('sally', 'sue');
                """)
                conn.commit()
                self.assertTrue(source.parse_source({
                    'name': 'example_source',
                    'job': 'example_job',
                    'exe_env': 'general',
                    'source': 'sql_source',
                    'type': 'postgresql',
                    'dsn': 'GREASE_TEST_DSN',
                    'query': 'select * from test_data;',
                    'hour': (datetime.datetime.utcnow() + datetime.timedelta(hours=6)).hour,
                    'minute': (datetime.datetime.utcnow() + datetime.timedelta(minutes=10)).minute,
                    'logic': {}
                }))
                data = source.get_data()
                self.assertTrue(isinstance(data, list))
                self.assertEqual(len(data), 0)
                cursor.execute("""
                    DROP TABLE public.test_data
                """)
        self.__cleanup_schema()



from unittest import TestCase
from tgt_grease.enterprise.Detectors import Exists


class TestExists(TestCase):

    def test_emptyList(self):
        e = Exists()
        state, data = e.processObject({}, [])
        self.assertFalse(state)
        self.assertFalse(data)

    def test_valid(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'field': 'test',
                    'variable': True,
                    'variable_name': 'test1'
                },
                {
                    'field': 'ver',
                }
            ]
        )
        self.assertTrue(state)
        self.assertEqual(data.get('test1'), 'data')

    def test_empty_field(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': '',
                'data': 5
            },
            [
                {
                    'field': 'ver',
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_valid_no_variables(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'field': 'test'
                },
                {
                    'field': 'ver'
                },
                {
                    'field': 'data'
                }
            ]
        )
        self.assertTrue(state)
        self.assertEqual(len(data), 0)

    def test_invalid_config(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                },
                {
                    'field': 'data',
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_failed_exists(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'pattern': 'test1'
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_bad_source(self):
        e = Exists()
        state, data = e.processObject(
            [],
            [
                {
                    'field': 'test'
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_bad_config(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            {}
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_bad_block(self):
        e = Exists()
        state, data = e.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                []
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)



from tgt_grease.enterprise.Model import PrototypeConfig
from tgt_grease.core import GreaseContainer
from unittest import TestCase
import json
import fnmatch
import os
import time
import pkg_resources


class TestPrototypeConfig(TestCase):

    def test_type(self):
        conf = PrototypeConfig()
        self.assertTrue(isinstance(conf, object))

    def test_empty_conf(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        self.assertTrue(conf.getConfiguration())
        self.assertListEqual(conf.getConfiguration().get('configuration').get('pkg'), [])
        self.assertListEqual(conf.getConfiguration().get('configuration').get('fs'), [])
        self.assertListEqual(conf.getConfiguration().get('configuration').get('mongo'), [])
        conf.load(reloadConf=True)

    def test_validation_list_good(self):
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertEqual(configList, conf.validate_config_list(configList))
        conf.load(reloadConf=True)

    def test_validation_list_bad(self):
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        configList = [
            {
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {}
            },
            {
                "name": "test1",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": []
            },
            {
                "name": "test4",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {

                }
            },
            {
                "name": "test5",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "var",
                            "pattern": "ver.*"
                        }
                    ]
                }
            }
        ]
        CompareList = [
            {
                "name": "test5",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "var",
                            "pattern": "ver.*"
                        }
                    ]
                }
            }
        ]
        self.assertEqual(CompareList, conf.validate_config_list(configList))
        conf.load(reloadConf=True)

    def test_validation_good(self):
        test = {
            "name": "test5",
            "job": "fakeJob",
            "exe_env": "windows",
            "source": "swapi",
            "logic": {
                "regex": [
                    {
                        "field": "var",
                        "pattern": "ver.*"
                    }
                ]
            }
        }
        conf = PrototypeConfig()
        self.assertTrue(conf.validate_config(test))

    def test_validation_bad(self):
        conf = PrototypeConfig()
        test = {
            "name": "test1",
            "exe_env": "windows",
            "source": "swapi",
            "logic": {
                "regex": [
                    {
                        "field": "character",
                        "pattern": ".*skywalker.*"
                    }
                ]
            }
        }
        self.assertFalse(conf.validate_config(test))
        test1 = {
            "name": "test2",
            "job": "fakeJob",
            "exe_env": "windows",
        }
        self.assertFalse(conf.validate_config(test1))
        test2 = {
            "name": "test3",
            "job": "fakeJob",
            "exe_env": "windows",
            "source": "swapi",
            "logic": []
        }
        self.assertFalse(conf.validate_config(test2))
        test3 = {
            "name": "test4",
            "job": "fakeJob",
            "exe_env": "windows",
            "source": "swapi",
            "logic": {

            }
        }
        self.assertFalse(conf.validate_config(test3))

    def test_load_good(self):
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertDictEqual(
            {
                'raw': configList,
                'configuration': {
                    'ConfigurationList': configList
                },
                'source': {
                    'swapi': configList,
                },
                'sources': ['swapi'],
                'names': ['test1', 'test2', 'test3'],
                'name': {
                    'test1': configList[0],
                    'test2': configList[1],
                    'test3': configList[2]
                }
            },
            conf.load(ConfigurationList=configList)
        )
        conf.load(reloadConf=True)

    def test_load_bad(self):
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        configList = [
            {
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {}
            },
            {
                "name": "test1",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": []
            },
            {
                "name": "test4",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {

                }
            },
            {
                "name": "test5",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "var",
                            "pattern": "ver.*"
                        }
                    ]
                }
            }
        ]
        CompareList = [
            {
                "name": "test5",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "var",
                            "pattern": "ver.*"
                        }
                    ]
                }
            }
        ]
        self.assertDictEqual(
            {
                'raw': CompareList,
                'configuration': {
                    'ConfigurationList': CompareList
                },
                'source': {
                    'swapi': CompareList
                },
                'sources': ['swapi'],
                'names': ['test5'],
                'name': {
                    'test5': CompareList[0]
                }
            },
            conf.load(ConfigurationList=configList)
        )
        conf.load(reloadConf=True)

    def test_get_sources_all_good(self):
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        conf.load(ConfigurationList=configList)
        self.assertEqual(['swapi', 'stackOverflow', 'Google'], conf.get_sources())
        conf.load(reloadConf=True)

    def test_get_sources_bad(self):
        conf = PrototypeConfig()
        conf.load(reloadConf=True)
        configList = [
            {
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {}
            },
            {
                "name": "test1",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": []
            },
            {
                "name": "test4",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "CarlBurger",
                "logic": {

                }
            },
            {
                "name": "test5",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "slack",
                "logic": {
                    "regex": [
                        {
                            "field": "var",
                            "pattern": "ver.*"
                        }
                    ]
                }
            }
        ]
        conf.load(ConfigurationList=configList)
        self.assertEqual(['slack'], conf.get_sources())
        conf.load(reloadConf=True)

    def test_fs_load_good(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        i = 0
        for conf in configList:
            with open(ioc.getConfig().get('Configuration', 'dir') + 'conf{0}.config.json'.format(i), 'w') as fil:
                fil.write(json.dumps(conf, indent=4))
            i += 1
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('fs')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('stackOverflow')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(3, len(conf.get_sources()))
        self.assertEqual(3, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('stackOverflow')), 1)
        self.assertTrue(isinstance(conf.get_config('test2'), dict))
        self.assertTrue(conf.get_config('test2'))
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clear the config
        conf.load(reloadConf=True)

    def test_fs_load_bad(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "badtest1",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        GoodConfigList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        i = 0
        for conf in configList:
            with open(ioc.getConfig().get('Configuration', 'dir') + 'conf{0}.config.json'.format(i), 'w') as fil:
                fil.write(json.dumps(conf, indent=4))
            i += 1
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('fs')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(2, len(conf.get_sources()))
        self.assertEqual(2, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('Google')), 1)
        self.assertTrue(isinstance(conf.get_config('test1'), dict))
        self.assertTrue(conf.get_config('test1'))
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clear the config
        conf.load(reloadConf=True)

    def test_pkg_load_good(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        i = 0
        for conf in configList:
            with open(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/') + 'conf{0}.config.json'.format(i), 'w') as fil:
                fil.write(json.dumps(conf, indent=4))
            i += 1
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('pkg')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('stackOverflow')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(3, len(conf.get_sources()))
        self.assertEqual(3, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('stackOverflow')), 1)
        self.assertTrue(isinstance(conf.get_config('test2'), dict))
        self.assertTrue(conf.get_config('test2'))
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clear the config
        conf.load(reloadConf=True)

    def test_pkg_load_bad(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "badtest1",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        GoodConfigList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        i = 0
        for conf in configList:
            with open(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/') + 'conf{0}.config.json'.format(i), 'w') as fil:
                fil.write(json.dumps(conf, indent=4))
            i += 1
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('pkg')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(2, len(conf.get_sources()))
        self.assertEqual(2, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('Google')), 1)
        self.assertTrue(isinstance(conf.get_config('test1'), dict))
        self.assertTrue(conf.get_config('test1'))
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clear the config
        conf.load(reloadConf=True)

    def test_mongo_load_good(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        for conf in configList:
            ioc.getCollection('Configuration').insert_one(conf)
        ioc.getCollection('Configuration').update_many({}, {'$set': {'active': True, 'type': 'prototype_config'}})
        # sleep because travis is slow
        time.sleep(1.5)
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('mongo')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('stackOverflow')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(3, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('stackOverflow')), 1)
        self.assertTrue(isinstance(conf.get_config('test2'), dict))
        self.assertTrue(conf.get_config('test2'))
        # clean up
        ioc.getCollection('Configuration').drop()
        # clear the config
        conf.load(reloadConf=True)

    def test_mongo_load_bad(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "badtest1",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        GoodConfigList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        for conf in configList:
            ioc.getCollection('Configuration').insert_one(conf)
        ioc.getCollection('Configuration').update_many({}, {'$set': {'active': True, 'type': 'prototype_config'}})
        # sleep because travis is slow sometimes
        time.sleep(1.5)
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('mongo')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(2, len(conf.get_sources()))
        self.assertEqual(2, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('Google')), 1)
        self.assertTrue(isinstance(conf.get_config('test1'), dict))
        self.assertTrue(conf.get_config('test1'))
        # clean up
        ioc.getCollection('Configuration').drop()
        # clear the config
        conf.load(reloadConf=True)

    def test_all_load_good(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        i = 0
        length = len(configList) - 1
        while i <= length:
            if i == 0:
                with open(ioc.getConfig().get('Configuration', 'dir') + 'conf{0}.config.json'.format(i), 'w') as fil:
                    fil.write(json.dumps(configList[i], indent=4))
            if i == 1:
                with open(pkg_resources.resource_filename('tgt_grease.enterprise.Model',
                                                          'config/') + 'conf{0}.config.json'.format(i), 'w') as fil:
                    fil.write(json.dumps(configList[i], indent=4))
            if i == 2:
                ioc.getCollection('Configuration').insert_one(configList[i])
            i += 1
        ioc.getCollection('Configuration').update_many({}, {'$set': {'active': True, 'type': 'prototype_config'}})
        # sleep because travis is slow
        time.sleep(1.5)
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('mongo')), 1)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('pkg')), 1)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('fs')), 1)
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(configList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('stackOverflow')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(3, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('stackOverflow')), 1)
        self.assertTrue(isinstance(conf.get_config('test2'), dict))
        self.assertTrue(conf.get_config('test2'))
        # clean up
        ioc.getCollection('Configuration').drop()
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clear the config
        conf.load(reloadConf=True)

    def test_all_load_bad(self):
        ioc = GreaseContainer()
        # clean up
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "badtest1",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        GoodConfigList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        i = 0
        length = len(configList) - 1
        while i <= length:
            if i == 0:
                with open(ioc.getConfig().get('Configuration', 'dir') + 'conf{0}.config.json'.format(i), 'w') as fil:
                    fil.write(json.dumps(configList[i], indent=4))
            if i == 1:
                with open(pkg_resources.resource_filename('tgt_grease.enterprise.Model',
                                                          'config/') + 'conf{0}.config.json'.format(i), 'w') as fil:
                    fil.write(json.dumps(configList[i], indent=4))
            if i == 2:
                ioc.getCollection('Configuration').insert_one(configList[i])
            i += 1
        ioc.getCollection('Configuration').update_many({}, {'$set': {'active': True, 'type': 'prototype_config'}})
        # sleep because travis is slow
        time.sleep(1.5)
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('mongo')), 1)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('pkg')), 0)
        self.assertEqual(len(conf.getConfiguration().get('configuration').get('fs')), 1)
        self.assertEqual(len(conf.getConfiguration().get('raw')), len(GoodConfigList))
        self.assertEqual(len(conf.getConfiguration().get('source').get('swapi')), 1)
        self.assertEqual(len(conf.getConfiguration().get('source').get('Google')), 1)
        self.assertEqual(2, len(conf.get_names()))
        self.assertEqual(len(conf.get_source('Google')), 1)
        self.assertTrue(isinstance(conf.get_config('test1'), dict))
        self.assertTrue(conf.get_config('test1'))
        # clean up
        ioc.getCollection('Configuration').drop()
        for root, dirnames, filenames in os.walk(ioc.getConfig().get('Configuration', 'dir')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clean up
        for root, dirnames, filenames in os.walk(pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                self.assertIsNone(os.remove(os.path.join(root, filename)))
        # clear the config
        conf.load(reloadConf=True)



from tgt_grease.enterprise.Model import BaseSourceClass
from tgt_grease.core import Configuration
import elasticsearch
import os
import fnmatch
import datetime
import json


class ElasticSource(BaseSourceClass):
    """Source data from ElasticSearch

    This Source is designed to query ElasticSearch for data. A generic configuration looks like this for a
    elastic_source::

        {
            'name': 'example_source', # <-- A name
            'job': 'example_job', # <-- Any job you want to run
            'exe_env': 'general', # <-- Selected execution environment; Can be anything!
            'source': 'elastic_source', # <-- This source
            'server': 'http://localhost:9200', # <-- String for ES Connection to occur
            'index': 'my_fake_index', # <-- Index to query within ES
            'doc_type': 'myData' # <-- Document type to query for in ES
            'query': {}, # <-- Dict of ElasticSearch Query
            'hour': 16, # <-- **OPTIONAL** 24hr time hour to poll SQL
            'minute': 30, # <-- **OPTIONAL** Minute to poll SQL
            'logic': {} # <-- Whatever logic your heart desires
        }

    Note:
        without `minute` parameter the engine will poll for the entire hour
    Note:
        **Hour and minute parameters are in UTC time**
    Note:
        To only poll once an hour only set the **minute** field

    """

    def parse_source(self, configuration):
        """This will make a ElasticSearch connection & query to the configured server

        Args:
            configuration (dict): Configuration of Source. See Class Documentation above for more info

        Returns:
            bool: If True data will be scheduled for ingestion after deduplication. If False the engine will bail out

        """
        if configuration.get('hour'):
            if datetime.datetime.utcnow().hour != int(configuration.get('hour')):
                # it is not the correct hour
                return True
        if configuration.get('minute'):
            if datetime.datetime.utcnow().minute != int(configuration.get('minute')):
                # it is not the correct hour
                return True
        if configuration.get('server') \
                and configuration.get('query') \
                and configuration.get('index') \
                and configuration.get('doc_type'):
            try:
                es = elasticsearch.Elasticsearch(
                    "".join(configuration.get('server')),
                    timeout=30,
                    max_retries=2,
                    retry_on_timeout=True
                )
            except BaseException:
                # Failed to connect to ES
                return False
            try:
                self._data = es.search(
                    index=''.join(configuration.get('index')),
                    doc_type=''.join(configuration.get('doc_type')),
                    body=configuration.get('query')
                )
            except elasticsearch.ImproperlyConfigured:
                # Improperly configured request
                return False
            except elasticsearch.ElasticsearchException:
                # generic exception
                return False
            del es
        else:
            # Invalid parameters
            return False

    def mock_data(self, configuration):
        """Data from this source is mocked utilizing the GREASE Filesystem

        Mock data for this source can be place in `<GREASE_DIR>/etc/*.mock.es.json`. This source will pick up all these
        files and load them into the returning object. The data in these files should reflect what you expect to return
        from ElasticSearch

        Args:
            configuration (dict): Configuration Data for source

        Note:
            Argument **configuration** is not honored here

        Returns:
            list[dict]: Mocked Data

        """
        intermediate = list()
        matches = []
        conf = Configuration()
        for root, dirnames, filenames in os.walk(conf.greaseDir + 'etc'):
            for filename in fnmatch.filter(filenames, '*.mock.es.json'):
                matches.append(os.path.join(root, filename))
        for doc in matches:
            with open(doc) as current_file:
                content = current_file.read().replace('\r\n', '')
            try:
                intermediate.append(json.loads(content))
            except ValueError:
                continue
        return intermediate



from tgt_grease.core import Logging
from tgt_grease.core.Connectivity import Mongo
from datetime import datetime
from bson.objectid import ObjectId
import platform
import os


class GreaseContainer(object):
    """Inversion of Control Container for objects in GREASE"""

    _logger = None
    _mongo = None

    def __init__(self, Logger=None):
        if Logger and isinstance(Logger, Logging):
            self._logger = Logger
        else:
            self._logger = Logging()
        self._mongo = Mongo(self._logger.getConfig())

    def getLogger(self):
        """Get the logging instance

        Returns:
            Logging: The logging instance

        """
        return self._logger

    def getNotification(self):
        """Get the notifications instance

        Returns:
            tgt_grease.core.Notifications: The notifications instance

        """
        return self._logger.getNotification()

    def getMongo(self):
        """Get the Mongo instance

        Returns:
            Mongo: Mongo Instance Connection

        """
        return self._mongo

    def getCollection(self, collectionName):
        """Get a collection object from MongoDB

        Args:
            collectionName (str): Collection to get

        Returns:
            pymongo.collection.Collection: Collection instance

        """
        return self.getMongo()\
            .Client()\
            .get_database(self.getConfig().get('Connectivity', 'MongoDB').get('db', 'grease'))\
            .get_collection(collectionName)

    def getConfig(self):
        """Gets the Configuration Instance

        Returns:
            tgt_grease.core.Configuration.Configuration: the configuration instance

        """
        return self._logger.getConfig()

    def ensureRegistration(self):
        """

        :return:
        """
        collection = self.getCollection("JobServer")
        if os.path.isfile(self.getConfig().greaseDir + 'grease.identity'):
            # check to see if identity file is valid
            fil = open(self.getConfig().greaseDir + 'grease.identity', 'r')
            nodeId = "".join(fil.read())
            fil.close()
            server = collection.find_one({'_id': ObjectId(nodeId)})
            if server:
                # Valid registration
                self.getConfig().NodeIdentity = nodeId
                return True
            else:
                self.getLogger().warning("Invalid node identity found to exist!")
        if self.getConfig().NodeIdentity == "Unknown":
            # Actual registration
            uid = collection.insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["monitor"],
                'active': True,
                'activationTime': datetime.utcnow()
            }).inserted_id
            fil = open(self.getConfig().greaseDir + "grease.identity", "w")
            fil.write(str(uid))
            fil.close()
            self.getConfig().NodeIdentity = uid
            del collection
            return True
        else:
            # Check the Identity is actually registered
            if collection.find({'_id': ObjectId(self.getConfig().NodeIdentity)}).count():
                del collection
                return True
            else:
                self.getLogger().error("Invalid Node Identity::Node Identity Not Found", additional={
                    'NodeID': self.getConfig().NodeIdentity
                })
                del collection
                return False



from tgt_grease.core.Types import Command
from tgt_grease.management.Model import NodeMonitoring
import random
import time


class ClusterMonitor(Command):
    """Cluster Monitor to ensure nodes are alive"""

    __author__ = "James E. Bell Jr."
    __version__ = "2.0.0"
    purpose = "Control cluster health"
    help = """
    Ensures health of GREASE cluster by providing a Prototype to
    scan the active nodes and disable those that are unhealthy
    
    Args:
        --loop:<int>
            How many scans you would like to perform of cluster
        --foreground
            If set will print log messages to the commandline
    
    """

    def __init__(self):
        super(ClusterMonitor, self).__init__()

    def execute(self, context):
        """This method monitors the environment

        An [in]finite loop monitoring the cluster nodes for unhealthy ones

        Args:
            context (dict): context for the command to use

        Returns:
            bool: Command Success

        """
        if context.get('foreground'):
            self.ioc.getLogger().foreground = True
        monitor = NodeMonitoring(self.ioc)
        if context.get('loop'):
            i = 0
            while i < int(context.get('loop', 0)):
                if not monitor.monitor():
                    self.ioc.getLogger().error("Monitoring Process Failed", notify=False)
                else:
                    monitor.scanComplete()
                i += 1
                time.sleep(5)
        else:
            while True:
                if not monitor.monitor():
                    self.ioc.getLogger().error("Monitoring Process Failed", notify=False)
                else:
                    monitor.scanComplete()
                # sleep for a random interval to ensure not all nodes poll at the same time
                time.sleep(int(str(random.choice(range(1, 5))) + str(random.choice(range(0, 9)))))
        if context.get('foreground'):
            self.ioc.getLogger().foreground = False
        return True



from unittest import TestCase
from tgt_grease.core.Types import Command
from tgt_grease.core import GreaseContainer
import sys


class TestCmd(Command):
    def __init__(self):
        super(TestCmd, self).__init__()

    def execute(self, context):
        for key, val in context.items():
            self.setData(key, val)
        return True


class TestCmdFail(Command):
    def __init__(self):
        super(TestCmdFail, self).__init__()

    def execute(self, context):
        for key, val in context.items():
            self.setData(key, val)
        return False


class TestCmdExcept(Command):
    def __init__(self):
        super(TestCmdExcept, self).__init__()

    def execute(self, context):
        for key, val in context.items():
            self.setData(key, val)
        raise AttributeError


class TestCommand(TestCase):
    def test_ioc(self):
        cmd = TestCmd()
        self.assertTrue(isinstance(cmd.ioc, GreaseContainer))

    def test_variable_storage_name(self):
        cmd = TestCmd()
        self.assertEqual(cmd.variable_storage.name, "TestCmd")

    def test_variable_storage(self):
        cmd = TestCmd()
        cmd.variable_storage.insert_one({'test': 'value'})
        self.assertEqual(cmd.variable_storage.find({'test': 'value'}).count(), 1)
        self.assertTrue(cmd.variable_storage.delete_one({'test': 'value'}))

    def test_get_exec_info(self):
        cmd = TestCmd()
        cmd.safe_execute({})
        self.assertTrue(isinstance(cmd.getExecVal(), bool))

    def test_get_ret_val_info(self):
        cmd = TestCmd()
        cmd.safe_execute({})
        self.assertTrue(isinstance(cmd.getRetVal(), bool))

    def test_get_ret_data(self):
        cmd = TestCmd()
        cmd.safe_execute({'key': 'value'})
        self.assertDictEqual(cmd.getData(), {'key': 'value'})

    def test_failed_cmd(self):
        cmd = TestCmdFail()
        cmd.safe_execute({})
        self.assertTrue(cmd.getExecVal())
        self.assertFalse(cmd.getRetVal())

    def test_except_cmd(self):
        def msg(m, additional=None):
            if additional is None:
                additional = {}
            self.assertEqual(
                m,
                "Failed to execute [TestCmdExcept] execute got exception!"
            )
            if sys.version_info[0] < 3:
                self.assertDictEqual(
                    additional,
                    {
                        'file': 'test_command.py',
                        'type': "<type 'exceptions.AttributeError'>",
                        'line': '34'
                    }
                )
            else:
                self.assertDictEqual(
                    additional,
                    {
                            'file': 'test_command.py',
                            'type': "<class 'AttributeError'>",
                            'line': '34'
                    }
                )
        cmd = TestCmdExcept()
        cmd.ioc._logger.error = msg
        cmd.safe_execute({})
        self.assertFalse(cmd.getExecVal())
        self.assertFalse(cmd.getRetVal())



from abc import ABCMeta, abstractmethod
from tgt_grease.core import GreaseContainer


class Detector(object):
    """Base Detection Class

    This is the abstract class for detectors to implement

    Attributes:
        ioc (GreaseContainer): IOC Access

    """

    __metaclass__ = ABCMeta

    def __init__(self, ioc=None):
        if isinstance(ioc, GreaseContainer):
            self.ioc = ioc
        else:
            self.ioc = GreaseContainer()

    @abstractmethod
    def processObject(self, source, ruleConfig):
        """Processes an object and returns valid rule data

        Data returned in the second parameter from this method should be in this form::

            {
                '<field>': Object # <-- if specified as a variable then return the key->Value pairs
                ...
            }

        Args:
            source (dict): Source Data
            ruleConfig (list[dict]): Rule Configuration Data

        Return:
            tuple: first element boolean for success; second dict for any fields returned as variables

        """
        pass



from tgt_grease.core.Types import Command
from tgt_grease.enterprise.Model import Detect


class Detection(Command):
    """The detect command

    This class is the source detection for GREASE. It utilizes the logic block of your configurations to determine
    if a job needs to be run

    """

    __author__ = "James E. Bell Jr."
    __version__ = "2.0.0"
    purpose = "Detect sources from scan and schedule them for job scheduling"
    help = """
    This command detects possible jobs from the scan command
    
    Args:
        --loop:<int>
            How many detection cycles to do
        --foreground
            Print log messages to the foreground
    """

    def execute(self, context):
        """Execute method of the detection prototype

        Args:
            context (dict): Command Context

        Note:
            This method normally will *never* return. As it is a prototype. So it should continue into infinity

        Returns:
             bool: True always unless failures occur

        """
        if context.get('foreground'):
            # set foreground if in context
            self.ioc.getLogger().foreground = True
        Detector = Detect(self.ioc)
        if 'loop' in context:
            # scan only a certain amount of times
            scan_count = 0
            while scan_count < int(context.get('loop')):
                if not Detector.detectSource():
                    self.ioc.getLogger().warning("Detection Process Failed", notify=False)
                scan_count += 1
        else:
            try:
                while True:
                    if not Detector.detectSource():
                        self.ioc.getLogger().warning("Detection Process Failed", notify=False)
                    continue
            except KeyboardInterrupt:
                # graceful close for scanning
                self.ioc.getLogger().trace("Keyboard interrupt in detect detected", trace=True)
                return True
        # ensure we clean up after ourselves
        if context.get('foreground'):
            self.ioc.getLogger().foreground = False
        return True



from unittest import TestCase
from tgt_grease.enterprise.Detectors import Range


class TestRange(TestCase):

    def test_range_compare_min_pass(self):
        r = Range()
        self.assertTrue(r.range_compare(5, {'min': 4}))
        self.assertTrue(r.range_compare(4.1, {'min': 4.0}))

    def test_range_compare_min_fail(self):
        r = Range()
        self.assertFalse(r.range_compare(3, {'min': 4}))
        self.assertFalse(r.range_compare(3.9, {'min': 4.0}))

    def test_range_compare_max_pass(self):
        r = Range()
        self.assertTrue(r.range_compare(3, {'max': 4}))
        self.assertTrue(r.range_compare(3.9, {'max': 4.0}))

    def test_range_compare_max_fail(self):
        r = Range()
        self.assertFalse(r.range_compare(5, {'max': 4}))
        self.assertFalse(r.range_compare(4.1, {'max': 4.0}))

    def test_range_compare_min_and_max_pass(self):
        r = Range()
        self.assertTrue(r.range_compare(3, {'max': 4, 'min': 1}))
        self.assertTrue(r.range_compare(3.9, {'max': 4.0, 'min': 1}))

    def test_range_compare_min_and_max_fail(self):
        r = Range()
        self.assertFalse(r.range_compare(5, {'max': 4, 'min': 1}))
        self.assertFalse(r.range_compare(4.1, {'max': 4.0, 'min': 1}))

    def test_range_compare_no_min_or_max(self):
        r = Range()
        self.assertFalse(r.range_compare(5, {}))

    def test_range_compare_bad_field_type(self):
        r = Range()
        self.assertFalse(r.range_compare('cats', {'max': 4, 'min': 1}))

    def test_range_compare_bad_min_or_max(self):
        r = Range()
        self.assertFalse(r.range_compare(3, {'min': 'cat'}))
        self.assertFalse(r.range_compare(3.9, {'min': 'dog'}))
        self.assertFalse(r.range_compare(5, {'max': 'cat'}))
        self.assertFalse(r.range_compare(4.1, {'max': 'dog'}))
        self.assertFalse(r.range_compare(5, {'max': 'cat', 'min': 'bagel'}))
        self.assertFalse(r.range_compare(4.1, {'max': 'dog', 'min': 'bagel'}))

    def test_emptyList(self):
        r = Range()
        state, data = r.processObject({}, [])
        self.assertFalse(state)
        self.assertFalse(data)

    def test_valid(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'field': 'data',
                    'min': 4,
                    'variable': True,
                    'variable_name': 'data'
                }
            ]
        )
        self.assertTrue(state)
        self.assertEqual(data.get('data'), 5)

    def test_valid_no_variables(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'field': 'data',
                    'min': 5,
                }
            ]
        )
        self.assertTrue(state)
        self.assertEqual(len(data), 0)

    def test_invalid_config(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                },
                {
                    'pattern': '.*(var).*'
                },
                {
                    'field': 'data',
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_failed_regex(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'max': 4
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_bad_source(self):
        r = Range()
        state, data = r.processObject(
            [],
            [
                {
                    'field': 'data',
                    'min': 3
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_bad_config(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            {}
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_empty_result(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'field': 'data',
                    'min': 7
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_field_not_exist(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                {
                    'field': 'data2',
                    'min': 6
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_field_false(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': False
            },
            [
                {
                    'field': 'data',
                    'min': 6
                }
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)

    def test_bad_block(self):
        r = Range()
        state, data = r.processObject(
            {
                'test': u'data',
                'ver': 'var',
                'data': 5
            },
            [
                []
            ]
        )
        self.assertFalse(state)
        self.assertEqual(len(data), 0)



from unittest import TestCase
from tgt_grease.core import GreaseContainer, Logging, Configuration, Notifications
from tgt_grease.core.Connectivity import Mongo
from pymongo.collection import Collection


class TestIOC(TestCase):
    def test_logger(self):
        ioc = GreaseContainer()
        self.assertTrue(isinstance(ioc.getLogger(), Logging))

    def test_config(self):
        ioc = GreaseContainer()
        self.assertTrue(isinstance(ioc.getConfig(), Configuration))

    def test_notifications(self):
        ioc = GreaseContainer()
        self.assertTrue(isinstance(ioc.getNotification(), Notifications))

    def test_mongo(self):
        ioc = GreaseContainer()
        self.assertTrue(isinstance(ioc.getMongo(), Mongo))

    def test_get_collection(self):
        ioc = GreaseContainer()
        self.assertTrue(isinstance(ioc.getMongo(), Mongo))
        coll = ioc.getCollection('TestCollection')
        self.assertTrue(isinstance(coll, Collection))
        self.assertEqual(coll.name, "TestCollection")

    def test_registration(self):
        ioc = GreaseContainer()
        self.assertTrue(ioc.ensureRegistration())



import os
import json

##
# Global Configuration Object
##
GREASE_CONFIG = None


class Configuration(object):
    """GREASE Configuration Management

    This class is responsible for management of configuration of a GREASE Node. Default Configuration will be
    used if the grease.conf.json document is not found in the root of the GREASE directory. It will ensure all
    folders/files are in the directory and serve as the access point for configuration data

    Attributes:
        greaseDir (str): The root directory of GREASE
        fs_sep (str): The filesystem separator for the installed OS
        greaseConfigFile (str): Location of the current configuration file
        FileSystem (list): Directories of the GREASE filesystem
        GREASE_CONFIG (dict): Actual config

    """
    global GREASE_CONFIG

    if os.environ.get('GREASE_DIR'):
        greaseDir = os.environ.get('GREASE_DIR')
    else:
        if os.name == 'nt':
            greaseDir = "C:\\grease\\"
        else:
            greaseDir = "/opt/grease/"

    FileSystem = ['etc', 'log']

    fs_sep = os.sep

    greaseConfigFile = greaseDir + "grease.conf.json"

    if os.path.isfile(greaseDir + 'grease.identity'):
        with open(greaseDir + 'grease.identity', 'r') as fil:
            NodeIdentity = fil.read().rstrip()
    else:
        NodeIdentity = "Unknown"

    def __init__(self, ConfigFile=None):
        self.EnsureGreaseFS()
        self.ReloadConfig(ConfigFile)

    @staticmethod
    def ReloadConfig(ConfigFile=None):
        """[Re]loads the configuration

        Returns:
            None: Void Method

        """
        global GREASE_CONFIG
        if ConfigFile:
            Configuration.greaseConfigFile = ConfigFile
        if os.path.isfile(Configuration.greaseConfigFile):
            # attempt loading
            try:
                fil = open(Configuration.greaseConfigFile)
                GREASE_CONFIG = json.loads(fil.read())
                fil.close()
            except:
                # load defaults
                GREASE_CONFIG = Configuration.DefaultConfig()
        else:
            # load defaults
            GREASE_CONFIG = Configuration.DefaultConfig()
            # write config to disk
            try:
                with open(Configuration.greaseConfigFile, 'w') as fil:
                    fil.write(json.dumps(GREASE_CONFIG, indent=4, sort_keys=True))
            except:
                print("""
!!!!!!!!!!!!!!!!!!FAILED TO WRITE CONFIGURATION!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                
THIS IS CRITICAL! EITHER [{0}] is unavailable or does not exist! log files will not be written and only the
default configuration is accessible!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                """.format(Configuration.greaseDir))
                return

    @staticmethod
    def get(section, key=None, default=None):
        """Retrieve configuration item

        Args:
            section (str): Configuration Section to read from
            key (str): Configuration key to retrieve
            default (object): Default value if section/key is not found

        """
        global GREASE_CONFIG
        if isinstance(GREASE_CONFIG, dict):
            if GREASE_CONFIG.get(section):
                if key:
                    if isinstance(GREASE_CONFIG.get(section), dict) and key in GREASE_CONFIG.get(section):
                        return GREASE_CONFIG.get(section).get(key)
                    else:
                        return default
                else:
                    return GREASE_CONFIG.get(section)
            else:
                return default
        else:
            return False

    def set(self, key, value, section=None):
        """Set configuration item

        Args:
            section (str): Configuration Section to set
            key (str): Configuration key to set
            value (object): value to set

        Returns:
            None: Sets item only

        """
        global GREASE_CONFIG
        if section:
            if isinstance(GREASE_CONFIG, dict):
                if GREASE_CONFIG.get(section):
                    GREASE_CONFIG[section][key] = value
                else:
                    GREASE_CONFIG[section] = dict()
                    GREASE_CONFIG[section][key] = value
            else:
                return False
        else:
            GREASE_CONFIG[key] = value

    #########
    # Initialization / Internal Methods
    #########

    def EnsureGreaseFS(self):
        """Ensures the GREASE Directory structure is setup

        Returns:
            bool: If the FS is in place then True

        """
        try:
            if not os.path.isdir(self.greaseDir):
                os.mkdir(self.greaseDir)
            for elem in self.FileSystem:
                if not os.path.isdir(self.greaseDir + elem):
                    os.mkdir(self.greaseDir + elem)
            return True
        except:
            return False

    @staticmethod
    def DefaultConfig():
        """Returns the Default GREASE Config

        Returns:
            dict: Default Configuration

        """
        return {
            'Connectivity': {
                'MongoDB': {
                    'host': 'localhost',
                    'port': 27017
                }
            },
            'Logging': {
                'mode': 'filesystem',
                'verbose': False,
                'trace': False,
                'foreground': False,
                'file': Configuration.greaseDir + 'log' + os.sep + 'grease.log'
            },
            'Notifications': {
                'HipChat': {
                    'enabled': False,
                    'token': None,
                    'room': None
                },
                'Slack': {
                    'enabled': False,
                    'webhookURL': ''
                }
            },
            'Configuration': {
                'dir': Configuration.greaseDir + 'etc' + os.sep
            },
            'Sourcing': {
                'dir': Configuration.greaseDir + 'etc' + os.sep,
                'source': None,
                'config': None,
                'mock': False
            },
            'Import': {
                'searchPath': [
                    'tgt_grease.router.Commands',
                    'tgt_grease.enterprise.Prototype',
                    'tgt_grease.management.Commands',
                    'tgt_grease.enterprise.Sources',
                    'tgt_grease.enterprise.Detectors',
                    'tgt_grease.core',
                    'tgt_grease'
                ]
            },
            "NodeInformation": {
                "ResourceMax": 95,
                "DeduplicationThreads": 150
            },
            "Additional": {}
        }



from .Command import Command
import datetime
from abc import ABCMeta, abstractmethod
from bson.objectid import ObjectId
from tgt_grease.enterprise.Model import Deduplication


class ScheduledCommand(Command):
    """Scheduled Commands Run as Prototypes

    This type is used for commands that need to be run cyclically. They will be run as prototypes (always running).
    Make sure to fill out the **timeToRun** and **run** methods

    """

    __metaclass__ = ABCMeta

    def __init__(self, logger=None):
        super(ScheduledCommand, self).__init__(logger)

    @abstractmethod
    def timeToRun(self):
        """Checks to ensure it is time to run

        Returns:
            bool: If time to run then true else false

        """
        pass

    @abstractmethod
    def run(self):
        """Put your code here to run whenever the conditions in timeToRun are defined

        Note:
            We recommend returning something valuable since the engine logs the result of the method in verbose mode

        """
        pass

    def execute(self, context):
        """Command execute method

        This will run continuously waiting for `timeToRun` to return true to call `run`

        Args:
            context (dict): context for the command to use **Not Used Here**

        """
        while True:
            if self.timeToRun():
                result = self.run()
                self.ioc.getLogger().debug(
                    "Scheduled command result of [{0}] was [{1}]".format(
                        self.__class__.__name__,
                        result
                    ), verbose=True
                )
                self.ioc.getLogger().info("Scheduled Command [{0}] has executed".format(self.__class__.__name__))
                self.ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(self.ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(self.ioc.getConfig().NodeIdentity),
                            'start': datetime.datetime.utcnow(),
                            'end': datetime.datetime.utcnow(),
                            'detection': {}
                        },
                        'scheduling': {
                            'server': ObjectId(self.ioc.getConfig().NodeIdentity),
                            'start': datetime.datetime.utcnow(),
                            'end': datetime.datetime.utcnow()
                        },
                        'execution': {
                            'server': ObjectId(self.ioc.getConfig().NodeIdentity),
                            'assignmentTime': datetime.datetime.utcnow(),
                            'completeTime': datetime.datetime.utcnow(),
                            'returnData': {},
                            'executionSuccess': True,
                            'commandSuccess': True,
                            'failures': 0
                        }
                    },
                    'source': 'grease_internal_scheduled_job_{0}'.format(self.__class__.__name__),
                    'configuration': None,
                    'data': {},
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
                })
            else:
                self.ioc.getLogger().trace(
                    "Scheduled Job not scheduled to run [{0}]".format(self.__class__.__name__),
                    trace=True
                )
                continue
